<?php

class Base
{
	private static $settings = array();
	private static $models = array();
	private static $request = array(); // User request array
	
	public function __construct() {}
	
	/*
	 * Load a library
	 * @param string $name Library name
	 * @param string $type Either core, model or lib
	 */
	public static function loadLib($name, $type = null)
	{
		if (is_null($type))
			require_once APPLICATION.'core/'.$name.'.php';
		else if ($type == 'libs')
			require_once APPLICATION.'libs/'.$name.'.php';
	}
	
	/*
	 * Parse settings
	 * $file string $file Path to the settings ini file
	 */
	public static function setSettings($file)
	{
		require_once $file;
	}
	
	/*
	 * Get a setting
	 * @param string $key Setting name
	 * return mixed
	 */
	public static function get($key)
	{
		if (isset(self::$settings[$key]))
		{
			return self::$settings[$key];
		}
		else
			throw new Exception('Setting "'.$key.'" does not exist');
	}
	
	/*
	 * Set a setting
	 * @param string $key Name of a setting
	 * @param mixed $val Value
	 */
	public static function set($key, $val)
	{
		if (!isset(self::$settings[$key]))
			self::$settings[$key] = $val;
		else
			throw new Exception('Setting "'.$key.'" is already set');
	}
	
    /*
	 * Returns a model of a module
	 * @param $key string The name of the module
	 * @return object Model class
	 */
	public static function getModel($key)
	{
		if (!isset(self::$models[$key]))
		{
			$file = MODULES.$key.'/model.php';
			if (file_exists($file))
			{
				require_once($file);
				$model = ucfirst($key);
				self::$models[$key] = new $model(self::$settings['db']);
			}
			else
				throw new Exception('Model does not exist');
		}
		return self::$models[$key];
	}
	
	/*
	 * Get the value of a request variable if it exists
	 * @param $key string Request variable
	 * @return string | null if var not exists
	 */
	public static function getRequest($key)
	{
		if (isset(self::$request[$key]))
			return self::$request[$key];
		return null;
	}
	
	/*
	 * Parse user's request string into an array
	 */
	public static function parseRequest($requestStr)
	{
		// Prepare request array
		if (substr($requestStr, -1) == '/') $requestStr = substr($requestStr, 0, -1);
		$request = array();
		if ($requestStr != '') $request = explode('/', $requestStr);
		
		// Apply plugin filter
		$request = do_filter('parse_request', $request);
		
		// Define module and action
		$request['module'] = isset($request[0]) ? preg_replace('/[^a-zA-Z0-9\-_]/', '', $request[0]) : 'index';
		$request['action'] = isset($request[1]) ? preg_replace('/[^a-zA-Z0-9\-_]/', '', $request[1]) : 'index';
		unset($request[0], $request[1]);
		
		// Save request options
		$items = count($request);
		if ($items > 2)
		{
			$next_item = 0;
			$before_last = $items - 1;
			for ($i = 2; $i < $before_last; ++$i)
			{
				$next_item = $i + 1;
				if ($next_item < $items)
				{
					$request[$request[$i++]] = $request[$next_item];
				}
			}
			if ($items % 2 == 1)
				$request[$request[$before_last]] = true;
		}
		
		// Save request within this class
		self::$request = $request;
		
		return $request;
	}
}