<?php

class FileCache
{
  private $_cacheDir;
  
  public function __construct($cacheDir)
  {
    $this->_cacheDir = $cacheDir;
  }
  
  public function get($key, $time)
  {
    $file = $this->_cacheDir.$key.'.cache';
    if (file_exists($file))
    {
      $fileTime = filemtime($file) + $time;
      $expireTime = $_SERVER['REQUEST_TIME'];
      
      if ($fileTime > $expireTime)
        return file_get_contents($file);
    }
    return false;
  }
  
  public function set($key, $val)
  {
    $file = $this->_cacheDir.$key.'.cache';
    return file_put_contents($file, $val, LOCK_EX);
  }
  
  public function delete($key)
  {
    $file = $this->_cacheDir.$key.'.cache';
    @unlink($file);
  }
  
  public function deleteLike($key)
  {
    $key = str_replace('..', '', $key);
    $dir = opendir($this->_cacheDir);
    while ($file = readdir($dir))
    {
      if ($file == '.' || $file == '..') continue;
      
      if (strpos($file, $key) === 0)
      {
        $file = $this->_cacheDir.$file;
        @unlink($file);
      }
    }
  }
}

class Cache
{
  private $_driver;
  private $_fragment;
  private $_nocache;
  
  public function __construct($driver, $param = null, $nocache = false)
  {
    $this->_driver = new $driver($param);
    $this->_nocache = $nocache;
  }
  
  public function setCache($key, $val = null)
  {
    if ($this->_nocache) return false;
     
    return $this->_driver->set($key, $val);
  }
  
  public function getCache($key, $time)
  {
    if ($this->_nocache) return false;
    
    return $this->_driver->get($key, $time);
  }
  
  public function setFragment()
  {
    if (!$this->_fragment || $this->_nocache) return false;
    
    $val = ob_get_contents();
    ob_end_clean();
    
    echo $val;
    
    return $this->_driver->set($this->_fragment, $val);
  }
  
  public function getFragment($key, $time)
  {
    if ($this->_nocache) return false;
    
    if (!$key)
    {
      $this->_fragment = null;
      return false;
    }
    
    $this->_fragment = $key;
    $cache = $this->_driver->get($key, $time);
    if ($cache === false)
    {
      ob_start();
      return false;
    }
    echo $cache;
    return true;
  }
  
  public function delete($key, $action = null)
  {
    if ($action == 'like')
      $this->_driver->deleteLike($key);
    else
      $this->_driver->delete($key);
  }
}
