<?php

/*
 * MySQL database manipulation class
 * 
 * Author: Dmytro Danylov (dime3studio)
 * Date: 24/10/2009
 * Update: 07/07/2010
 */
class Db
{
	private $link;
	private $numQueries;
	private $sql;
	
	/*
	 * Connect to database
	 * @param array $settings Database credentials
	 * @throws Exception
	 */
	public function __construct($settings, $permanent = false, $charset = 'utf8')
	{	
		// Connect to database
		if ($permanent)
			$this->link = mysql_pconnect($settings['host'], $settings['user'], $settings['pass']);
		else
			$this->link = mysql_connect($settings['host'], $settings['user'], $settings['pass']);
			
		if (!$this->link)
			exit('Could not connect to database');
			
		// Set charset
		mysql_query('SET NAMES "'.$charset.'"', $this->link);
		
		// Select database
		if (!mysql_select_db($settings['dbname'], $this->link))
			exit('Could not select database');
	}
	
	/*
	 * Query
	 * @param string $sql
	 */
	public function query($sql)
	{
		++$this->numQueries;
		$this->sql[] = $sql;
		$result = mysql_query($sql, $this->link);
		// Log query and error
		/*$error = mysql_error();
		$logdata = <<<EOT
QUERY:
$sql
ERROR:
$error
EOT;
		// Save to log
		file_put_contents(APPLICATION.'logs/db.txt', $logdata, FILE_APPEND);*/
		return $result;
	}
	
	public function select($sql)
	{
		$result = $this->query($sql);
		if (!$result || $this->numRows($result) == 0)
			return false;
		return $result;
	}
	
	/*
	 * Fetch a single row as Associative array
	 * mixed $result Database query result 
	 */
	public function fetchAssoc($result)
	{
		return mysql_fetch_assoc($result);
	}
	
	/*
	 * Fetch associative array or object from result obtained by query() method
	 * @return array on success
	 * @return boolean on failure
	 */
	public function fetch($sql, $dataType = 'assoc')
	{
		$result = $this->query($sql);
		if ($result && $this->numRows($result) > 0)
		{
			$array = array();
			if ($dataType == 'assoc')
			{
				while ($row = mysql_fetch_assoc($result))
					$array[] = $row;
			}
			else
			{
				while ($row = mysql_fetch_object($result))
					$array[] = $row;
			}
			return $array;
		}
		return false;
	}
	
	/*
	 * Fetch one field from result obtained by query() method
	 * @return array on success
	 * @return boolean on failure
	 */
	public function fetchResult($sql, $row = 0, $field = 0)
	{
		$result = $this->query($sql);
		if ($result != false && $this->numRows($result) > 0)
			return mysql_result($result, $row, $field);
		return false;
	}
	
	/*
	 * Fetch number of returned rows
	 * @return int
	 */
	public function numRows($result)
	{
		return mysql_num_rows($result);
	}
	
	/*
	 * Fetch number of affected rows
	 * @return int
	 */
	public function affectedRows()
	{
		return mysql_affected_rows($this->link);
	}
	
	/*
	 * Prepare values for SQL queries 
	 */
	public function escape($mixed)
	{
		if (is_array($mixed))
		{
			foreach ($mixed as $key => $value)
				$mixed[$key] = $this->escape($value);
		}
		else
			$mixed = mysql_real_escape_string($mixed, $this->link);
		return $mixed;
	}
	
	/*
	 * Get ID of the last insert
	 */
	public function getInsertId()
	{
		return mysql_insert_id($this->link);
	}
	
	/*
	 * Get error number
	 */
	public function getErrNo()
	{
		return mysql_errno($this->link);
	}
	
	/*
	 * Close database connection
	 */
	public function close()
	{
		mysql_close($this->link);
	}
	
	public function insert($table, $data)
	{
		$sql = 'INSERT INTO '.$table.' (';
		$vals = ' VALUES (';
		$lastIdx = count($data) - 1;
		$i = 0;
		
		foreach ($data as $key => $val)
		{
			// Filter
			if (in_array($val, array('now()'))) {}
			else if (is_string($val))
				$val = '"'.$this->escape($val).'"';
			else if (!is_numeric($val))
				$val = 'null';
			
			$sql .= $key;
			$vals .= $val;
			
			if ($i++ < $lastIdx)
			{
				$sql .= ',';
				$vals .= ',';
			}
		}
		$result = $this->query($sql.') '.$vals.')');
		if ($result && $this->affectedRows() != 0)
			return $this->getInsertId();
		return false;
	}
	
	public function update($table, $data, $where)
	{
		$sql = 'UPDATE '.$table.' SET ';
		$lastIdx = count($data) - 1;
		$i = 0;
		
		foreach ($data as $key => $val)
		{
			// Filter
			if (in_array($val, array('now()'))) {}
			else if (is_string($val))
				$val = '"'.$this->escape($val).'"';
			else if (!is_numeric($val))
				$val = 'null';
			
			$sql .= $key.'='.$val;
			
			if ($i++ < $lastIdx)
				$sql .= ',';
		}
		
		return $this->query($sql.' WHERE '.$where);
	}
	
	public function showStats()
	{
		if (count($this->sql) > 0)
		{
			$report = <<<EOT
			<h4 style="margin-bottom:5px;">Database usage statistics:</h4>
			<table style="font-size: 10px;border-collapse:collapse;">
				<tr>
					<th style="padding:3px;border:1px solid #b0b0b0;">Order</th>
					<th style="padding:3px;border:1px solid #b0b0b0;">Query</th>
				</tr>
EOT;
			$count = 1;
			foreach ($this->sql as $sql)
			{
				$report .= <<<EOT
				<tr>
					<td style="padding:3px;border:1px solid #b0b0b0;">$count</td>
					<td style="padding:3px;border:1px solid #b0b0b0;">$sql</td>
				</tr>
EOT;
				++$count;
			}
			
			$report .= <<<EOT
				<tr>
					<td>Total</td>
					<td><b style="font-size:14px;">$this->numQueries</b> queries</td>
				</tr>
			</table>
EOT;
			
			echo $report;
		}
	}
}

?>