<?php

/*
 * Get file extension
 * @param string $filename
 * @return string
 */
function file_ext($filename) 
{ 
  $filename = strtolower($filename); 
  $exts = explode('.', $filename); 
  $n = count($exts) - 1;  
  return $exts[$n];
}

/*
 * Get file type based on extention
 * @param string $filename
 * @return string
 */
function file_type($filename)
{
  $ext = file_ext($filename);
  switch ($ext)
  {
    case 'jpg':
    case 'jpeg':
    	return 'image/jpeg';
    case 'png':
    	return 'image/png';
    case 'gif':
    	return 'image/gif';
    case 'tiff':
    	return 'image/tiff';
    
    case 'pdf':
    	return 'application/pdf';
    case 'doc':
    case 'docx':
    	return 'application/msword';
    case 'zip':
    	return 'application/zip';
    case 'txt':
    	return 'text/plain';
    
    case 'avi':
    case 'wmv':
    	return 'video/x-msvideo';
    case 'mov':
    	return 'video/quicktime';
  }
  return 'unknown';
}

function filter_filename($filename)
{
	$filename = preg_replace('/(\\|\/|\.\.)/ui', '', $filename);
	return $filename;
}

function save_file($file, $path)
{
	// Check if file exists
	if (file_exists($path))
		return 'exists';
	
	// Copy
	if (!move_uploaded_file($file, $path))
		return 'error';
		
	return 'success'; // if file is copied
}

/*
 * Get file upload error message
 * @param const int $error_code
 * @return string on error | false on success
 */
function file_error($error_code)
{
	switch ($error_code)
	{
		case UPLOAD_ERR_OK:
			return 'no_error';
			break;
			
		case UPLOAD_ERR_INI_SIZE:
		case UPLOAD_ERR_FORM_SIZE:
			return 'The uploaded file is too large';
			break;
			
		case UPLOAD_ERR_PARTIAL:
			return 'The uploaded file was only partially uploaded';
			break;
			
		case UPLOAD_ERR_NO_FILE:
			return 'No file was uploaded';
			break;
			
		case UPLOAD_ERR_NO_TMP_DIR:
			return 'Missing a temporary folder';
			break;
			
		case UPLOAD_ERR_CANT_WRITE:
			return 'Failed to write file to disk';
			break;
			
		//case UPLOAD_ERR_EXTENSION:
		default:
			return false;
	}
}

/*
 * Serve the file to the client
 * @param string $file Filename
 * @param string $path Path to the file including filename
 * @return void
 */
function serve_file($file, $path)
{
	if (file_exists($path))
	{
		// Serve the file
		header("Content-Type: application/force-download");
		header("Content-Length: ".filesize($path));
		header('Content-Disposition: attachment; filename="'.$file.'"');
		
		$fp = fopen($path, 'r');
		while (!feof($fp))
			echo fgets($fp, 10240); // serve file 10KB at a time
		fclose($fp);
	}
	else
		echo '<h2>File does not exist</h2>';
}