<?php

/***************************************************************************** MESSAGING FUNCTIONS */
/*
 * Save a text message in the session
 * @param string $message
 */
function set_flash_message($message, $type = null)
{
  switch ($type)
  {
  case 'error':
    $messageWrap = '<p class="error">%s</p>';
    break;
    
  case 'success':
    $messageWrap = '<p class="success">%s</p>';
    break;
    
  case 'warning':
    $messageWrap = '<p class="warning">%s</p>';
    break;
    
  default:
    $messageWrap = '%s';
  }
  $_SESSION['FLASH_MESSAGE'] = sprintf($messageWrap, $message);
}

/*
 * Pass a text message from one script to another 
 */
function flash_message()
{
  if (isset($_SESSION['FLASH_MESSAGE']))
  {
    echo $_SESSION['FLASH_MESSAGE'];
    unset($_SESSION['FLASH_MESSAGE']);
  }
}

/*
 * Show breadcrumbs
 */
function breadcrumbs($currModule, $title)
{
  $module = Base::getRequest('module');
  $action = Base::getRequest('action');
  echo '
  <ul class="breadcrumbs">
    <li><a href="',href($module.'/'.$action),'">',$currModule,'</a></li>
    <li><span>&gt;&gt;</span></li>
    <li><span>',$title,'</span></li>
  </ul>';
}

/***************************************************************************** URL */
function href($address, $admin=null)
{
  $base_url = Base::get('base_url');
  
  if (IS_ADMIN && $admin === null)
  {
    $base_url .= Base::get('admin').'.php';
  }

  if (PRETTY_URLS == 'n' || IS_ADMIN)
    return $base_url.'?q='.$address;
  else
    return $base_url.$address;

}

function getCurrentUrl()
{
  static $url = null;
  if (!$url)
  {
    $url = 'http://'.$_SERVER['SERVER_NAME'].$_SERVER['REQUEST_URI'];
    $url = filter_var($url, FILTER_SANITIZE_URL, FILTER_SANITIZE_STRING);
  }
  return $url;
}

function getFromUrl()
{
  static $from = null;
  if (is_null($from))
  {
    $from = Base::getRequest('from');
    if (!$from)
    {
      if (isset($_GET['from']))
        $from = $_GET['from'];
      else if (isset($_POST['from']))
        $from = $_POST['from'];
      else
        $from = null;
    }
    $from = filter_var($from, FILTER_SANITIZE_URL, FILTER_SANITIZE_STRING);
  }
  return $from;
}

/***************************************************************************** OPTIONS */
/*
 * Get site option from database
 * @param string $key Option name
 * return string | null on failure
 */
function get_option($key)
{
  static $options = null;
  
  $db = Base::get('db');
  
  if (is_null($options))
  {
    $options = array();
    $result = $db->select('select optionKey, optionVal from '.DB_PREFIX.'options
      WHERE deleted is null and autoload="y"');
    if ($result)
    {
      while($row = $db->fetchAssoc($result))
      {
        $options[$row['optionKey']] = $row['optionVal'];
      }
    }
  }
  
  if (!array_key_exists($key, $options))
  {
    $val = $db->fetchResult('select optionVal from '.DB_PREFIX.'options
      WHERE optionKey="'.$db->escape($key).'" and deleted is null');
    if ($val)
      $options[$key] = $val;
    else
      $options[$key] = null;
      
    return $options[$key];
  }
  else
    return $options[$key];
    
  return null;
}

/*
 * Add/Update option
 * @param string $key Option name
 * @param string $val Option value
 * @param char $autoload y or n
 * @return bool
 */
function set_option($key, $val, $autoload = 'n')
{
  $db = Base::get('db');
  
  $key = $db->escape($key);
  $val = $db->escape($val);
  
  $sql = 'insert into '.DB_PREFIX.'options (optionKey, optionVal, autoload)
    values ("'.$key.'", "'.$val.'", "'.$autoload.'")
    ON DUPLICATE KEY UPDATE optionVal="'.$val.'", autoload="'.$autoload.'"';
  
  return $db->query($sql);
}

/*
 * Delete option
 * @param string $key Option name
 * @return bool
 */
function del_option($key)
{
  $db = Base::get('db');
  
  $key = $db->escape($key);
  
  $sql = 'update '.DB_PREFIX.'options set deleted=now() where optionKey="'.$key.'"';
  return $db->query($sql);
}

/***************************************************************************** REDIRECTION */
function redirect($to)
{
  header('Location: '.$to);
  exit();
}

/***************************************************************************** SECURITY */
/*
 * Filter a variable to safely output in html
 * @param string $val
 */
function filterXss($val)
{
  return htmlspecialchars($val, ENT_NOQUOTES, 'utf-8', false);
}

/*
 * Filter an array to safely output in html
 * @param mixed $mixed
 */
function filterArrayXss(&$mixed)
{
  if (is_array($mixed))
  {
    foreach ($mixed as $key => $value)
      $mixed[$key] = htmlspecialchars($value, ENT_NOQUOTES, 'utf-8', false);
  }
  else
    $mixed = htmlspecialchars($mixed, ENT_NOQUOTES, 'utf-8', false);
}

/*
 * Strip injection chars from email headers
 * @param string $string
 * @return string
 */
function safeEmail($string)
{
  return  preg_replace('((?:\n|\r|\t|%0A|%0D|%08|%09)+)i', '', $string);
}

/*
 * Set security token, used to prevent CSRF
 * @param int $timeout Seconds
 */
function set_token($timeout = false)
{
  $salt = Base::get('salt');
  if ($timeout == false)
  {
    if (!isset($_SESSION['TOKEN']))
      $_SESSION['TOKEN'] = md5(uniqid(rand().$salt, true));
    return;
  }
  
  $tokenTime = time();
  if (!isset($_SESSION['TOKEN_TIME']) || $tokenTime - $_SESSION['TOKEN_TIME'] > 1800)
  {
    $_SESSION['TOKEN'] = md5(uniqid(rand().$salt, true));
    $_SESSION['TOKEN_TIME'] = $tokenTime;
  }
}

/*
 * Returns token from the session
 */
function get_token()
{
  return $_SESSION['TOKEN'];
}

/***************************************************************************** TEMPLATES */
function template($do = 'get', $tpl = null)
{
  static $template = null;
  if ($do == 'set')
    $template = $tpl;
  else
    return $template;
}

function htmlSelect($name, $items, $attr = '', $allowEmpty = true)
{
  $isset = isset($_POST[$name]);
  $attr = ' '.$attr;
  echo '<select name="',$name,'"',$attr,'>';
  if ($allowEmpty)
    echo '<option value="">--Select--</option>';
  foreach ($items as $key => $val)
  {
    echo '<option value="',$key,'"';
    if ($isset && $_POST[$name] == $key)
      echo ' selected="selected"';
    echo '>',$val,'</option>';
  }
  echo '</select>';
}

function formFieldValue($key, $replace = null)
{
  if (isset($_REQUEST[$key])) return filterXss($_REQUEST[$key]);
  return '';
}

function menuItem($title, $url, $active = null)
{
  if ($title == $active) $class = ' class="active"';
  else $class = '';
  
  echo '<a href="',$url,'"',$class,'>',$title,'</a>';
}

$siteMeta = array();

function setMeta($k, $d)
{
  global $siteMeta;
  $siteMeta['k'] = $k;
  $siteMeta['d'] = $d;
}

function showMeta()
{
  global $siteMeta;
  if (!isset($siteMeta['k'])) $siteMeta['k'] = get_option('metaKeywords');
  if (!isset($siteMeta['d'])) $siteMeta['d'] = get_option('metaDescription');
  
  echo '<meta name="keywords" content="',$siteMeta['k'],'" />';
  echo '<meta name="description" content="',$siteMeta['d'],'" />';
}

function getThemes()
{
  $path = THEMES;
  $dir = opendir($path);
  $themes = array();
  while ($file = readdir($dir))
  {
    if ($file == '.' || $file == '..' || $file=='admin' || is_file($path.$file)) continue;
    $themes[] = $file;
  }
  closedir($dir);
  return $themes;
}

/*
 * Get all available templates in the current theme
 * @return array
 */
function getLayouts()
{
  $path = THEMES.get_option('theme').'/layouts';
  $dir = opendir($path);
  $layouts = array();
  while ($file = readdir($dir))
  {
    if ($file == '.' || $file == '..') continue;
    $layouts[$file] = $file;
  }
  closedir($dir);
  return $layouts;
}


/***************************************************************************** HELPERS */
function formatDate($timestamp)
{
  $format = get_option('dateFormat');
  if (!$format) $format = "j M Y \a\\t H:i";
  return date($format, $timestamp);
}

/***************************************************************************** BLOG HELPERS */
function prepare_posts()
{
  global $posts, $cache, $cacheKey, $filter, $page;
  
  if (!$cache->getFragment($cacheKey, 30))
  {
    $perpage = (int)get_option('postsPage');
    if ($perpage <= 0) $perpage = 10;
    $posts = Base::getModel('posts')->getPostsByPage($page, $perpage, $filter);
    if ($posts->items)
      return true;
  }
  return false;
}

function after_posts()
{
  global $cache;
  $cache->setFragment();
}

function have_posts()
{
  global $posts;
  if ($posts && $posts->total_items > 0) return true;
  return false;
}

function get_post()
{
  global $posts, $post, $db;
  
  if ($posts->items)
    return $db->fetchAssoc($posts->items);
  return false;
}

function posts_pagination()
{
  global $posts;
  static $links = null;
  if ($links == null)
    $links = $posts->getLinks('?page=');
  return $links;
}

function get_url()
{
  global $post;
  $url = null;
  if ($url === null)
    $url = href('posts/view/'.$post['url_tag'].'/'.$post['postID']);
  return $url;
}

function comments_open()
{
  global $post;
  if (isset($post['comments']))
    return $post['comments'];
  return 0;
}

function ebl_list_comments()
{
  global $comments, $cpage, $cperpage;
  $cpage = (int)$cpage;
  $offset = ($cpage - 1) * $cperpage;
  if (get_option('commentsOrder') == 'desc')
    krsort($comments);
  else
    ksort($comments);
  $CommentsWalk = new CommentsWalk();
  $CommentsWalk->output(array_slice($comments, $offset, $cperpage, true));
}

function comments_pagination($next=null, $prev=null)
{
  global $comments, $cpage, $cperpage;
  $cpage = (int)$cpage;
  $cperpage = (int)$cperpage;
  
  $totalComments = count($comments);
  $totalPages = ceil($totalComments / $cperpage);
  
  $uri = filterXss(preg_replace('/(\?|&)?cpage=[0-9]*/', '', $_SERVER['REQUEST_URI']));
  $uri .= '&';
  
  if (!$prev)
    $prev = __('Newer comments');
  
  if (!$next)
    $next = __('Older comments');
  
  echo '<p>';
  
  if ($cpage > 1)
  {
    ?>
    <a href="<?php echo $uri,'cpage='.($cpage-1); ?>"><?php echo $prev; ?></a>
    <?php
  }
  
  if ($cpage < $totalPages)
  {
    ?>
    <a href="<?php echo $uri,'cpage='.($cpage+1); ?>"><?php echo $next; ?></a>
    <?php
  }
  
  echo '</p>';
}

function isHome()
{
  global $isHome;
  if ($isHome === true) return true;
  else return false;
}