<?php

class Pager
{
	public $total_items = 0;
	public $items;
	public $total_pages = 0;
	public $page = 1;
	
	/*
	 * Initialize the object
	 * @param int $page
	 */
	public function __construct($page = 1)
	{
	  $this->page = $page;
	}
	
	/*
	 * Get items on page
	 * @param string $sql
	 * @param object $db My database abstraction class
	 * @param int $perpage Number of items on page
	 */
	public function fetchPage($sql, $db, $perpage = 20)
	{
		$result = $db->query($sql);
		if ($result)
			$this->total_items = $db->numRows($result);
		$this->total_pages = ceil($this->total_items / $perpage);
		$offset = ($this->page - 1) * $perpage;
		$this->items = $db->select($sql." LIMIT $offset, $perpage");
	}
	
	/*
	 * Get pages links
	 * @param string $delimiter Representation of a page in url e.g. "?page=" or "&p=" or "/p/"
	 * @param string $url
	 * @param int $links_number
	 */
	public function getLinks($delimiter, $url = null, $links_number = 5)
	{
    $links = '';
    
    if ($this->total_pages > 1)
    {
      if ($url == null)
      {
      	$url = 'http://'.$_SERVER['SERVER_NAME'].$_SERVER['REQUEST_URI'];
        $url = preg_replace('/'.preg_quote($delimiter, '/').'[0-9]{0,3}/i', '', $url);
      }
      
      // Previous and first links
      if ($this->page > $links_number + 1) {
          $links .= '<a href="'.$url.$delimiter.'1"><<</a> ';
          $links .= '<a href="'.$url.$delimiter.($this->page - 1).'"><</a> ';
      }

      // Links from the left
      if ($this->page > 1) {
          for ($i = $this->page - $links_number; $i <= $this->page - 1; $i++) {
              if ($i < 1) $i=1;
              $links .= '<a href="'.$url.$delimiter.$i.'">'.$i.'</a> ';
          }
      }

      // Current link
      $links .= '<strong>'.$this->page.'</strong> ';

      // Links to the right
      for ($i = $this->page + 1; $i <= $this->page + $links_number; $i++) {
          if ($i > $this->total_pages) break;
          $links .= '<a href="'.$url.$delimiter.$i.'">'.$i.'</a> ';
      }

      // Next and last page links
      if ($this->page < $this->total_pages - $links_number) {
          $links .= '<a href="'.$url.$delimiter.($this->page + 1).'">></a> ';
          $links .= '<a href="'.$url.$delimiter.$this->total_pages.'">>></a>';
      }
    }

    return $links;
	}
}

?>