<?php

$actions_tbl = array();

/*
 * Load plugins
 * @param array $plugins List of plugins names
 */
function load_plugins($plugins)
{
	foreach ($plugins as $plugin)
	{
	  $plugin = preg_replace('/(\\|\/|\.\.)/', '', $plugin);
		$file = PLUGINS.$plugin.'/'.$plugin.'.php';
		if (file_exists($file))
			require_once $file;
	}
	return true;
}

function get_plugin_info($filename)
{
  $info = array(
    'Name' => 'Unknown',
    'Plugin Url' => null,
    'Description' => 'Unknown',
    'Version' => 'Unknown',
    'Author' => 'Unknown',
    'Author Url' => null
  );
  
  $fp = @fopen($filename, 'r');
  if ($fp)
  {
    while(($buff = fgets($fp, 4096)) !== false)
    {
      // Parse the file for information
      if ($buff == '/*') continue;
      else if ($buff == '*/') break;
      
      foreach ($info as $key => $val)
      {
        $find = $key.':';
        if (strpos($buff, $find) !== false)
        {
          $info[$key] = trim(str_replace($find, '', $buff));
          break;
        }
      }
    }
    fclose($fp);
    return $info;
  }
}

/*
 * Get plugins
 * @return array $plugins
 */
function get_plugins()
{
	$plugins = array();
	$dir = opendir(PLUGINS);
	if ($dir)
	{
		while ($plugin = readdir($dir))
		{
			$file = PLUGINS.$plugin;
			if ($plugin == '.' || $plugin == '..' || is_file($file))
				continue;
			
			$file .= '/'.$plugin.'.php';
			if (file_exists($file))
			{
				$plugins[$plugin] = get_plugin_info($file);
			}
		}
		closedir($dir);
	}
	else
		throw new Exception('Could not open plugins folder');
	return $plugins;
}

function get_active_plugins()
{
  return (array)unserialize(get_option('activePlugins'));
}

function add_action($tag, $function, $priority = 10, $accept_args = 1)
{
	global $actions_tbl;
	if (isset($actions_tbl[$tag][$priority]))
	{
		foreach ($actions_tbl[$tag][$priority] as $action)
		{
			if ($action['fn'] == $function)
				return false;
		}
	}
	$actions_tbl[$tag][$priority][] = array('fn' => $function, 'accept_args' => $accept_args);
	return true;
}

function remove_action($tag, $fn, $priority = 10)
{
	global $actions_tbl;
	if (isset($actions_tbl[$tag][$priority]))
	{
		$count = count($actions_tbl[$tag][$priority]);
		for ($i = 0; $i < $count; $i++)
		{
			if ($actions_tbl[$tag][$priority][$i]['fn'] == $function)
			{
				unset($actions_tbl[$tag][$priority][$i]['fn'][$i]);
				return;
			}
		}
	}
	return;
}

function do_action($tag)
{
	global $actions_tbl;
	
	$args = array_slice(func_get_args(), 1);
	
	if (!isset($actions_tbl[$tag]))
		return;
	else
		ksort($actions_tbl[$tag]);
	
	foreach ($actions_tbl[$tag] as $priority => $actions)
	{
		foreach($actions as $action)
		{
			$fn_args = array_slice($args, 0, $action['accept_args']);
			call_user_func_array($action['fn'], $fn_args);
		}
	}
}

function do_filter($tag, $string)
{
	global $actions_tbl;
	
	$args = array_slice(func_get_args(), 1);
	
	if (!isset($actions_tbl[$tag]))
		return $string;
	else
		ksort($actions_tbl[$tag]);
	
	foreach ($actions_tbl[$tag] as $priority => $actions)
	{
		foreach($actions as $action)
		{
			$fn_args = array_slice($args, 0, $action['accept_args']);
			$string = call_user_func_array($action['fn'], $fn_args);
		}
	}
	
	return $string;
}