<?php

class View
{
	protected $vars = array();
	protected $layout;
	protected $themeDir;
	protected $themeUrl;
	protected $tpl;
	protected $content;
	protected $jsPaths = array();
	protected $jsNames = array();
	
	/*
	 * Set default settings
	 * @param array $opt ('layout', 'themeDir', 'themeUrl', 'tpl', 'tplDir')
	 */
	public function __construct($opt)
	{
		$this->layout = $opt['layout'];
		$this->themeDir = $opt['themeDir'];
		$this->themeUrl = $opt['themeUrl'];
		$this->tpl = $opt['tpl'];
	}
	
	/*
	 * Add javascript to queue
	 * @param string $name Must be very basic (e.g. if file is jQuery.min.js, $name must be jquery)
	 * @param string $path Absolute url to the file
	 */
	public function addJs($name, $path, $where = null)
	{
		if (in_array($name, $this->jsNames))
			return false;
		if ($where == 'before')
		{
			array_unshift($this->jsNames, $name);
			array_unshift($this->jsPaths, $path);
		}
		else
		{
			$this->jsNames[] = $name;
			$this->jsPaths[] = $path;
		}
	}
	
	/*
	 * Outputs javascript files
	 */
	public function loadJs()
	{
		foreach ($this->jsPaths as $path)
			echo '<script type="text/javascript" rel="javascript" src="'.$path.'"></script>';
	}
	
	/*
	 * Changes current template
	 * @param string $tpl Template filename (e.g. template.php)
	 */
	public function setTemplate($tpl)
	{
		$this->tpl = $tpl;
	}
	
	/*
	 * Get a template
	 * @param string $file Absolute path to the file
	 */
	public function getTemplate($file, $vars = null)
	{
		if (is_array($vars))
			extract($vars);
		if (file_exists($file))
		{
			ob_start();
			include_once $file;
			$tpl = ob_get_contents();
			ob_end_clean();
			return $tpl;
		}
		return null;
	}
	
	/*
	 * Execute template output with current settings
	 */
	public function run()
	{
		extract($this->vars);
		include_once $this->themeDir.$this->layout;
	}
	
	public function showContent()
	{
		extract($this->vars);
		if (file_exists($this->tpl))
		{
			include_once $this->tpl;
		}
	}
	
	/*
	 * Set template variable
	 * @param string $key Name of a variable
	 * @param mixed $val Value
	 */
	public function set($key, $val)
	{
		$this->vars[$key] = $val;
	}
	
	/*
	 * Identical to set() method
	 */
	public function __set($key, $val)
	{
		$this->vars[$key] = $val;
	}
	
	/*
	 * Get a template variable
	 * @param string $key Variable name
	 * return mixed
	 */
	public function __get($key)
	{
		if (isset($this->vars[$key]))
			return $this->vars[$key];
		return null;
	}
	
	public function menuItem($url, $title, $title_active)
	{
		if ($title == $title_active)
			echo $title;
		else
			echo '<a href="',$url,'">',$title,'</a>';
	}
	
	public function setContent(&$content)
	{
		$this->content = $content;
	}
	
	public function setLayout($layout)
	{
		$this->layout = $layout;
	}
}