<?php

// Setup caching object
require_once APPLICATION.'core/cache.php';
$cache = new Cache('FileCache', APPLICATION.'cache/');

$postsM = Base::getModel('posts');
$usersM = Base::getModel('users');

// Authenticate the user
usersAuth($usersM);

switch ($request['action'])
{
  /* VIEW ALL POSTS */
  case 'index':
    $access = null;
    if ($usersM->perm('posts_view_all'))
      $access = 'posts_view_all';
    
    $type = Base::getRequest('type');
    if (!in_array($type, array('post','page')))
      $type = 'post';
    
    $status = Base::getRequest('status');
    if (!in_array($status, array('approved','pending','draft','deleted')))
      $status = 'all';
    
    $userID = null;
    $view = Base::getRequest('view');
    if ($view == 'mine' || $access == null)
      $userID = $_SESSION['eblUserID'];
    else
      $view = 'All authors';
      
    $filter = array();
    $filter['userID'] = $userID;
    $filter['status'] = $status;
    
    if ($type == 'post')
    {
      $page = isset($_GET['page']) ? (int)$_GET['page'] : 1;
      $posts = $postsM->getPostsByPage($page, 20, $filter);
      $title = $active = __('View posts');
    }
    else
    {
      $posts = $postsM->getPages();
      template('set', $theme['tplPath'].'/pages.php');
      $title = $active = __('View pages');
    }
    
    $stats = $postsM->getPostsStats($type, $userID);
    break;
    
  /* EDIT POSTS */
  case 'edit-post':
    $type = Base::getRequest('type');
    if (!in_array($type, array('post','page'))) $type = 'post';
    
    $from = getFromUrl();
    if (!$from)
      $from = href('posts/index/type/'.$type);
    
    $categoriesOpt = $postsM->getCategories('selectBox');
    $usersOpt = $usersM->getByGroup(array('admin', 'author'), 'selectBox');
    $hideOpt = array(0 => __('Show'), 1 => __('Hide'));
    $commentsOpt = array(0 => __('Yes'), 1 => __('No'));
    $blockOpt = array(0 => __('Yes'), 1 => __('No'));
    $hiddenOpt = array(1 => __('Yes'), 0 => __('No'));
    $layoutOpt = getLayouts();
    
    $access = $usersM->perm('posts_edit');
    
    if (!$access)
    {
      $_POST['block'] = 0;
      $_POST['userID'] = $_SESSION['eblUserID'];
    }
    
    if (!isset($_POST['userID']) || $_POST['userID'] == 0)
      $_POST['userID'] = $_SESSION['eblUserID'];
    
    $postID = (int)Base::getRequest('postID');
    if ($postID)
    {
      $post = $postsM->getPost($postID);
      
      if ($post)
      {
        $type = $post['type'];
        
        // Check if the user has access to edit posts of others
        if (!$access && $post['userID'] != $_SESSION['eblUserID'])
        {
          $post = null;
          $postID = null;
          $_POST['userID'] = $_SESSION['eblUserID'];
        }
        // Check if the post is blocked from editing by others
        else if ($post['block'] == 1 && $post['userID'] != $_SESSION['eblUserID'])
        {
          $post = null;
          $postID = null;
        }
      }
    }
    
    $errors = array();
    $allowed = array('token', 'from', 'status', 'title', 'url_tag', 'url', 'pos', 'description', 'content',
      'meta_k', 'meta_d', 'image', 'parentID', 'hidden', 'layout', 'categoryID', 'userID', 'comments', 'block');
    $sent = array_keys($_POST);
    
    sort($allowed);
    sort($sent);
    
    if ($allowed == $sent && $_POST['token'] == get_token())
    {
      // Validate layout
      if (!empty($_POST['layout']) && !array_key_exists($_POST['layout'], $layoutOpt))
        $errors['layout'] = __('Please select a layout');
        
      // Validate title and url tag
      if ($_POST['title'] == '')
        $errors['title'] = __('Please enter title');
        
      if (!preg_match('/^[a-zA-Z0-9\-_+]+$/', $_POST['url_tag']))
        $errors['url_tag'] = __('Please enter url tag. Allowed characters:').' a-z 0-9 - _ +';
      
      if (count($errors) == 0)
      {
        // Filter
        if (!empty($_POST['url_tag']))
          $_POST['url_tag'] = preg_replace('/[^a-zA-Z0-9]/', '-', $_POST['url_tag']);
        else
          $_POST['url_tag'] = preg_replace('/[^a-zA-Z0-9]/', '-', $_POST['title']);
          
        $clean = array();
        foreach ($_POST as $key => $value)
        {
          if ($key != 'content')
            $clean[$key] = filterXss($db->escape($value));
        }
        
        if (!$access && $clean['status'] == 'approved')
          $clean['status'] = 'pending';
  
        // Clean HTML
        require_once ROOT.'ext/htmlpurifier/library/HTMLPurifier.auto.php';
        $config = HTMLPurifier_Config::createDefault();
        $config->set("Core.Encoding", 'utf-8');
        $config->set("Attr.EnableID", true);
        $config->set("HTML.SafeEmbed", true);
        $config->set("HTML.SafeObject", true);
        $purifier = new HTMLPurifier($config);
        $clean['content'] = $db->escape($purifier->purify($_POST['content']));
        
        if (!$postID)
          $sql = 'INSERT INTO '.DB_PREFIX.'posts (type, status, title, url_tag, url, pos, description,
            content, meta_k, meta_d, image, parentID, categoryID, userID, comments, block, hidden, layout, date_posted)
            VALUES ("%s", "%s", "%s", "%s", "%s", %d, "%s", "%s", "%s", "%s", "%s", %d, %d, %d, %d, %d, %d, "%s", CURRENT_TIMESTAMP)';
        else
          $sql = 'UPDATE '.DB_PREFIX.'posts SET type="%s", status="%s", title="%s", url_tag="%s", url="%s", pos=%d, description="%s",
            content="%s", meta_k="%s", meta_d="%s", image="%s", parentID=%d, categoryID=%d, userID=%d, comments=%d, block=%d, hidden="%d", layout="%s"
            WHERE postID='.$postID.' LIMIT 1';
            
        $sql = sprintf($sql, $type, $clean['status'], $clean['title'], $clean['url_tag'], $clean['url'], $clean['pos'],
          $clean['description'], $clean['content'], $clean['meta_k'], $clean['meta_d'], $clean['image'], $clean['parentID'],
          $clean['categoryID'], $clean['userID'], $clean['comments'], $clean['block'], $clean['hidden'], $clean['layout'], $type);
          
        unset($clean);
        
        $result = $db->query($sql);
        if ($result)
        {
          set_flash_message(__('Saved'), 'success');
          redirect(getFromUrl());
        }
        else
        {
          set_flash_message(__('Database error occured'), 'error');
        }
      }
      else
      {
        set_flash_message(__('Please correct errors below'), 'error');
      }
    }
    else if ($postID)
    {
      $_POST = $post;
    }
    else
    {
      $_POST = array('status' => 'draft');
    }
    
    $title = $active = __('Create '.__($type));
    break;
  
  /* PROCESS POSTS */
  case 'posts-process':
    $action = null;
    if (isset($_POST['doaction']) && isset($_POST['action']))
      $action = $_POST['action'];
    else if (isset($_POST['doaction2']) && isset($_POST['action2']))
      $action = $_POST['action2'];
  
    if (isset($_POST['token']) && isset($_POST['posts']) && is_array($_POST['posts']) && $action)
    {
      if ($_POST['token'] == get_token())
      {
        $ids = array();
        foreach ($_POST['posts'] as $postID)
        {
          if (is_numeric($postID))
          {
            $ids[] = $postID;
            // Delete cached comments for the selected posts
            $cache->delete('comments-'.$postID);
          }
        }
        
        if (!$usersM->perm('posts_edit'))
        {
          // Get the IDs of posts that belong to current user
          $userPostIDs = array();
          $result = $db->select('SELECT postID FROM '.DB_PREFIX.'posts WHERE userID='.(int)$_SESSION['eblUserID']);
          if ($result)
            while($row = $db->fetchAssoc($result))
              $userPostIDs[] = $row['postID'];
          $ids = array_intersect($ids, $userPostIDs);
          
          if ($action == 'approved')
          {
            set_flash_message(__('Access denied'), 'warning');
            redirect(getFromUrl());
          }
        }
        
        $idsStr = implode(',', $ids);
        
        switch ($action)
        {
          case 'approved':
          case 'pending':
          case 'draft':
          case 'deleted':
            $status = $action;
            
            if ($idsStr != '')
            {
              $sql = 'UPDATE '.DB_PREFIX.'posts SET status="'.$status.'" WHERE postID IN ('.$idsStr.')';
              $result = $db->query($sql);
            }
            else
            {
              set_flash_message(__('Access denied'), 'warning');
              redirect(getFromUrl());
            }
            
            break;
            
          case 'delete_permanently':
            usersCheckAccess('posts_delete');
            
            $sql = 'DELETE FROM '.DB_PREFIX.'posts WHERE postID IN ('.$idsStr.')';
            $result = $db->query($sql);
            
            break;
        }

        if ($result)
        {
          set_flash_message(__('Done'), 'success');
          // Delete post's cache
          $cache->delete('post', 'like');
        }
        else
          set_flash_message(__('Error occured'), 'error');
      }
    }

    redirect(getFromUrl());
    
    break;
  
  /* DELETE POSTS */
  case 'delete-post':
    usersCheckAccess('posts_delete');
    
    $result = null;
    
    if (Base::getRequest('token') == get_token())
    {
      $postID = (int)Base::getRequest('postID');
      $sql = 'DELETE FROM '.DB_PREFIX.'posts WHERE postID='.$postID.' LIMIT 1';
      $result = $db->query($sql);
    }
    
    if (!$result)
      set_flash_message(__('Error occured'), 'error');
    else
      set_flash_message(__('Complete'), 'success');
    
    $from = getFromUrl();
    if (!$from)
      $from = href('posts/index');
    
    redirect($from);
    break;
    
  /* VIEW CATEGORIES */
  case 'categories':
    usersCheckAccess('categories_edit');
    $categories = $postsM->getCategories();
    $title = __('Categories');
    $active = __('View categories');
    break;
    
  /* EDIT CATEGORIES */
  case 'edit-category':
    usersCheckAccess('categories_edit');
    
    $categoryID = (int)Base::getRequest('categoryID');
    
    if ($categoryID)
      $category = $postsM->getCategory($categoryID);
      
    $allowed = array('token', 'title', 'url_tag', 'pos');
      
    if ($allowed == array_keys($_POST) && $_POST['token'] == get_token())
    {
      // Validation
      $clean = array();
      $clean['title'] = filterXss($_POST['title']);
      $clean['url_tag'] = filterXss($_POST['url_tag']);
      $clean['pos'] = (int)$_POST['pos'];
      $clean = $db->escape($clean);
      
      if (!$categoryID)
        $sql = 'INSERT INTO '.DB_PREFIX.'categories (title, url_tag, pos) VALUES ("%s", "%s", %d)';
      else
        $sql = 'UPDATE '.DB_PREFIX.'categories SET title="%s", url_tag="%s", pos=%d WHERE categoryID='.$categoryID;
        
      $sql = sprintf($sql, $clean['title'], $clean['url_tag'], $clean['pos']);
        
      $result = $db->query($sql);
      if ($result)
      {
        set_flash_message(__('Saved'), 'success');
        redirect(href('posts/categories'));
      }
      else
      {
        set_flash_message(__('Database error occured'), 'error');
      }
    }
    else if ($categoryID)
    {
      $_POST = $category;
    }
    else
    {
      $_POST = array();
    }
    
    $title = __('Edit category');
    $active = __('Create category');
    break;
    
  /* DELETE CATEGORIES */
  case 'delete-category':
    usersCheckAccess('categories_delete');
    
    $result = null;
    
    if (Base::getRequest('token') == get_token())
    {
      $categoryID = (int)Base::getRequest('categoryID');
      $sql = 'DELETE FROM '.DB_PREFIX.'categories WHERE categoryID='.$categoryID.' LIMIT 1';
      $result = $db->query($sql);
    }
    
    if (!$result)
      set_flash_message(__('Error occured'), 'error');
    else
      set_flash_message(__('Complete'), 'success');
      
    redirect(href('posts/categories'));
    break;
    
  /* SORT CATEGORIES */
  case 'sort-categories':
    usersCheckAccess('categories_edit');
    
    foreach ($_POST['category'] as $pos => $id)
    {
      $sql = 'UPDATE '.DB_PREFIX.'categories SET pos='.(int)$pos.'
        WHERE categoryID='.(int)$id.' LIMIT 1';
      $db->query($sql);
    }
    exit();
    break;
  
  /* LIST LINKS */
  case 'links':
    $links = $postsM->getLinks();
    
    $active = $title = __('View links');
    break;
    
  /* EDIT CATEGORIES */
  case 'edit-link':
    usersCheckAccess('links_edit');
    
    $linkID = (int)Base::getRequest('linkID');
    
    if ($linkID)
      $link = $postsM->getLink($linkID);
      
    $allowed = array('token', 'title', 'url', 'pos');
      
    if ($allowed == array_keys($_POST) && $_POST['token'] == get_token())
    {
      // Validation
      $clean = array();
      $clean['title'] = filterXss($_POST['title']);
      $clean['url'] = filterXss($_POST['url']);
      $clean['pos'] = (int)$_POST['pos'];
      $clean = $db->escape($clean);
      
      if (!$linkID)
        $sql = 'INSERT INTO '.DB_PREFIX.'menu_links (title, url, pos) VALUES ("%s", "%s", %d)';
      else
        $sql = 'UPDATE '.DB_PREFIX.'menu_links SET title="%s", url="%s", pos=%d WHERE linkID='.$linkID;
        
      $sql = sprintf($sql, $clean['title'], $clean['url'], $clean['pos']);
        
      $result = $db->query($sql);
      if ($result)
      {
        set_flash_message(__('Saved'), 'success');
        redirect(href('posts/links'));
      }
      else
      {
        set_flash_message(__('Database error occured'), 'error');
      }
    }
    else if ($linkID)
    {
      $_POST = $link;
    }
    else
    {
      $_POST = array();
    }
    
    $title = __('Edit link');
    $active = __('Create link');
    break;
    
  /* DELETE CATEGORIES */
  case 'delete-link':
    usersCheckAccess('links_delete');
    
    $result = null;
    
    if (Base::getRequest('token') == get_token())
    {
      $linkID = (int)Base::getRequest('linkID');
      $sql = 'DELETE FROM '.DB_PREFIX.'menu_links WHERE linkID='.$linkID.' LIMIT 1';
      $result = $db->query($sql);
    }
    
    if (!$result)
      set_flash_message(__('Error occured'), 'error');
    else
      set_flash_message(__('Complete'), 'success');
      
    redirect(href('posts/links'));
    break;
    
  /* SORT LINKS */
  case 'sort-links':
    usersCheckAccess('links_edit');
    
    foreach ($_POST['link'] as $pos => $id)
    {
      $sql = 'UPDATE '.DB_PREFIX.'menu_links SET pos='.(int)$pos.'
        WHERE linkID='.(int)$id.' LIMIT 1';
      $db->query($sql);
    }
    exit();
    break;
    
  /* COMMENTS */
  case 'comments':
    $access = null;
    if ($usersM->perm('posts_comments_view'))
      $access = 'posts_comments_view';
    
    $userID = (int)$_SESSION['eblUserID'];
    $postID = (int)Base::getRequest('postID');
    $status = Base::getRequest('status');
    if (!$status) $status = 'all';
    $page = isset($_GET['page']) ? (int)$_GET['page'] : 1;
    
    // GET COMMENTS
    $sql = 'SELECT c1.commentID, c1.parentID, c1.postID, c1.categoryID, c1.name,
      c1.email, c1.email_hide, c1.content, UNIX_TIMESTAMP(c1.date_posted) as date_posted, c1.userID,
      c1.ip, c1.website, p.title as postTitle, p.url_tag as postUrlTag
      FROM '.DB_PREFIX.'comments c1
      LEFT JOIN '.DB_PREFIX.'posts p ON p.postID=c1.postID
      WHERE 1';
      
    // Filter by userID
    if ($access == null)
      $sql .= ' AND c1.userID='.$userID;
      
    // Filter by postID
    $postID = (int)$postID;
    if ($postID)
      $sql .= ' AND c1.postID='.$postID;
      
    // Filter by status
    if (in_array($status, array('approved','pending','spam', 'deleted')))
      $sql .= ' AND c1.status="'.$status.'"';
    else if ($status == 'all')
      $sql .= ' AND c1.status <> "deleted"';
      
    $sql .= ' ORDER BY c1.commentID DESC';
    
    require_once APPLICATION.'core/pager.php';
    $comments = new Pager($page);
    $comments->fetchPage($sql, $db, 10);
    // GET COMMENTS END
    
    if ($access == null)
      $stats = $postsM->getCommentsStats($userID);
    else
      $stats = $postsM->getCommentsStats();
      
    $title = $active = __('Comments');
    break;
    
  /* EDIT COMMENT */
  case 'edit-comment':
    $access = null;
    if ($usersM->perm('posts_comments_edit'))
      $access = 'posts_comments_edit';
      
    $from = getFromUrl();
    if (!$from)
      $from = href('posts/comments');
  
    $commentID = (int)Base::getRequest('commentID');
    if ($commentID)
    {
      $comment = $postsM->getComment($commentID);
      if ($comment)
      {
        if ($comment['userID'] != $_SESSION['eblUserID'])
          usersCheckAccess('posts_comments_edit');
        else
          if (isset($_POST['status']) && $_POST['status'] == 'approved') $_POST['status'] = 'pending';
      }
      else
      {
        redirect(getFromUrl());
      }
    }
    else
    {
      redirect(getFromUrl());
    }
    
    $errors = array();
    
    $allowed = array('token', 'from', 'status', 'name', 'email', 'website', 'content');
    
    if ($allowed == array_keys($_POST))
    {
      // Filter
      $clean = array();
      foreach ($_POST as $key => $val)
        $clean[$key] = $db->escape(filterXss($val));
        
      $sql = 'UPDATE '.DB_PREFIX.'comments SET name="%s", email="%s", website="%s", content="%s", status="%s"
        WHERE commentID='.$commentID.' LIMIT 1';
        
      $sql = sprintf($sql, $clean['name'], $clean['email'], $clean['website'], $clean['content'], $clean['status']);
      
      $result = $db->query($sql);
      
      if ($result)
      {
        set_flash_message(__('Saved'), 'success');
        redirect(getFromUrl());
      }
      else
      {
        set_flash_message(__('Error occured. You may try again.'));
      }
    }
    else if ($comment)
    {
      $_POST = $comment;
    }
    
    $title = __('Edit comment');
    $active = __('Comments');
    break;
    
  case 'comments-process':
    $action = null;
    if (isset($_POST['doaction']) && isset($_POST['action']))
      $action = $_POST['action'];
    else if (isset($_POST['doaction2']) && isset($_POST['action2']))
      $action = $_POST['action2'];
    
    if (isset($_POST['token']) && isset($_POST['comments']) && is_array($_POST['comments']) && $action)
    {
      if ($_POST['token'] == get_token())
      {
        $ids = array();
        foreach ($_POST['comments'] as $commentID)
        {
          if (is_numeric($commentID))
            $ids[] = $commentID;
        }
        
        if (!$usersM->perm('posts_comments_edit'))
        {
          // Get the IDs of posts that belong to current user
          $userCommentIDs = array();
          $result = $db->select('SELECT commentID FROM '.DB_PREFIX.'comments WHERE userID='.(int)$_SESSION['eblUserID']);
          if ($result)
            while($row = $db->fetchAssoc($result))
              $userCommentIDs[] = $row['commentID'];
          $ids = array_intersect($ids, $userCommentIDs);
          
          if ($action == 'approved')
          {
            set_flash_message(__('Access denied'), 'warning');
            redirect(getFromUrl());
          }
        }
        
        $idsStr = implode(',', $ids);

        switch ($action)
        {
          case 'approved':
          case 'pending':
          case 'spam':
          case 'deleted':
            $status = $action;
            
            if (count($ids) > 0)
            {
              $sql = 'UPDATE '.DB_PREFIX.'comments SET status="'.$status.'" WHERE commentID IN ('.$idsStr.')';
              $result = $db->query($sql);
            }
            break;
            
          case 'delete_permanently':
            usersCheckAccess('posts_comments_delete');
            
            $sql = 'DELETE FROM '.DB_PREFIX.'comments WHERE commentID IN ('.$idsStr.')';
            $result = $db->query($sql);
            
            break;
        }

        if ($result)
          set_flash_message(__('Done'), 'success');
        else
          set_flash_message(__('Error occured'), 'error');
      }
    }
    redirect(getFromUrl());
    
    break;
    
  default:
}
