<?php

class Posts
{
  private $_db;
  private $_categories = null;
  
  public function __construct($db)
  {
    $this->_db = $db;
  }
  
  public function getCategories($format = 'assoc')
  {
    $sql = 'SELECT categoryID, title, pos, url_tag FROM '.DB_PREFIX.'categories
      ORDER BY pos ASC';
    $result = $this->_db->select($sql);
    $data = array();
    if ($result)
    {
      if ($format == 'assoc')
      {
        while ($row = $this->_db->fetchAssoc($result))
          $data[$row['categoryID']] = $row;
      }
      else if ($format == 'selectBox')
      {
        while ($row = $this->_db->fetchAssoc($result))
          $data[$row['categoryID']] = $row['title'];
      }
      else
      {
        return $result;
      }
    }
    return $data;
  }
  
  public function getCategory($categoryID)
  {
    $categoryID = (int)$categoryID;
    $sql = 'SELECT categoryID, title, pos, url_tag FROM '.DB_PREFIX.'categories
      WHERE categoryID='.$categoryID.' LIMIT 1';
    $result = $this->_db->select($sql);
    if ($result)
      return $this->_db->fetchAssoc($result);
    return null;
  }
  
  public function getPost($postID, $all = false)
  {
    $sql = 'SELECT p.postID, p.parentID, p.userID, p.categoryID, p.type, p.status, p.title, p.description,
      p.meta_k, p.meta_d, p.content, p.image, p.comments, UNIX_TIMESTAMP(p.date_posted) as date_posted,
      p.block, p.url_tag, p.url, p.pos, p.layout, p.hidden, c.title as category, c.url_tag as categoryUrlTag,
      u.name as userName, u.username as userUsername
      FROM '.DB_PREFIX.'posts p
      LEFT JOIN '.DB_PREFIX.'categories c ON c.categoryID=p.categoryID
      LEFT JOIN '.DB_PREFIX.'users u ON u.userID=p.userID';
    
    if (!is_numeric($postID))
    {
      $postID = $this->_db->escape($postID);
      $sql .= ' WHERE p.url_tag="'.$postID.'" LIMIT 1';
    }
    else
    {
      $sql .= ' WHERE p.postID='.$postID.' LIMIT 1';
    }

    $result = $this->_db->select($sql);
    if ($result)
      return $this->_db->fetchAssoc($result);
    return null;
  }
  
  public function getPosts($type = 'recent', $qty = 5, $format = 'assoc')
  {
    $qty = (int)$qty;
    
    $sql = 'SELECT p.postID, p.userID, p.categoryID, p.status, p.title, p.description, p.content,
      p.image, p.comments, UNIX_TIMESTAMP(p.date_posted) as date_posted, p.block, p.url_tag, p.url, p.pos, p.layout, p.hidden,
      c.title as category, c.url_tag as categoryUrlTag, u.name as userName, u.username as userUsername
      FROM '.DB_PREFIX.'posts p
      LEFT JOIN '.DB_PREFIX.'categories c ON c.categoryID=p.categoryID
      LEFT JOIN '.DB_PREFIX.'users u ON u.userID=p.userID
      WHERE type="post" AND status="approved"';
      
    if ($type == 'recent')
      $sql .= ' ORDER BY pos DESC, postID DESC';
       
    $sql .= ' LIMIT '.$qty;
    
    $result = $this->_db->select($sql);
    if ($format == 'resource')
      return $result;
    $data = array();
    if ($result)
      while ($row = $this->_db->fetchAssoc($result))
        $data[$row['postID']] = $row;
    return $data;
  }
  
  public function getPostsByPage($page, $numPosts, $params = array())
  {
    extract($params);
    $numPosts = (int)$numPosts;
    
    $sql = 'SELECT p.postID, p.userID, p.categoryID, p.status, p.title, p.description, p.content,
      p.image, p.comments, UNIX_TIMESTAMP(p.date_posted) as date_posted, p.block, p.url_tag, p.url, p.pos, p.layout, p.hidden,
      c.title as category, c.url_tag as categoryUrlTag, u.name as userName, u.username as userUsername
      FROM '.DB_PREFIX.'posts p
      LEFT JOIN '.DB_PREFIX.'categories c ON c.categoryID=p.categoryID
      LEFT JOIN '.DB_PREFIX.'users u ON u.userID=p.userID
      WHERE 1';
    
    if (isset($status) && in_array($status, array('approved','pending','draft','deleted')))
      $sql .= ' AND status="'.$status.'"';
    else
      $sql .= ' AND status<>"deleted"';
    
    if (isset($userID) && $userID)
      $sql .= ' AND p.userID='.(int)$userID;
      
    if (isset($category))
    {
      if (is_string($category))
      {
        $category = $this->_db->escape($category);
        $sql .= ' AND c.url_tag="'.$category.'"';
      }
      else
      {
        $category = (int)$category;
        $sql .= ' AND c.categoryID="'.$category.'"';
      }
    }
    
    if (isset($date))
    {
      $date = $this->_db->escape($date);
      $sql .= ' AND p.date_posted LIKE "'.$date.'%"';
    }
    
    if (isset($search))
    {
      // Prepare keywords for search
      $search = $this->_db->escape($search);
      $search = explode(' ', $search);
      $count = count($search);
      $words = '';
      
      $num = 0;
      $score = '';
      for ($i = 0; $i < $count; ++$i)
      {
        if (strlen($search[$i]) > 2)
        {
          $words .= ' OR (p.title LIKE "%'.$search[$i].'%" OR p.meta_k LIKE "%'.$search[$i].'%")';
          ++$num;
        }
        // Only 6 keywords, to reduce server load
        if ($num > 5) break;
      }
      
      $words = '('.substr($words, 4).')';
      
      $sql .= ' AND '.$words;
    }
    else
    {
      $sql .= ' AND type="post"';
    }
    
    $sql .= ' ORDER BY pos DESC, postID DESC';
    
    require_once APPLICATION.'core/pager.php';
    $page = (int)$page;
    $pager = new Pager($page);
    $pager->fetchPage($sql, $this->_db, $numPosts);
    /*
    $result = $this->_db->query($sql);
    if ($result)
      $pager->total_items = $this->_db->numRows($result);
    $pager->total_pages = ceil($pager->total_items / $numPosts);
    $offset = ($page - 1) * $numPosts;
    $result = $this->_db->select($sql." LIMIT $offset, $numPosts");
    if ($result)
    {
      $pager->items = array();
      while ($row = $this->_db->fetchAssoc($result))
        $pager->items[$row['postID']] = $row;
    }
    else
      $pager->items = null;
    */
    return $pager;
  }
  
  public function getPages($params = array())
  {
    extract($params);
    
    $sql = 'SELECT p.postID, p.parentID, p.userID, p.categoryID, p.title, p.description, p.content,
      p.image, p.comments, p.date_posted, p.block, p.url_tag, p.status, p.url, p.pos, p.layout, p.hidden,
      c.title as category, u.name as userName, u.username as userUsername
      FROM '.DB_PREFIX.'posts p
      LEFT JOIN '.DB_PREFIX.'categories c ON c.categoryID=p.categoryID
      LEFT JOIN '.DB_PREFIX.'users u ON u.userID=p.userID
      WHERE p.type="page"';
    
    if (!IS_ADMIN)
      $sql .= ' AND status="approved"';
    
    if (isset($status) && in_array($status, array('approved','pending','draft','deleted')))
      $sql .= ' AND status="'.$status.'"';
      
    if (isset($userID) && $userID)
      $sql .= ' AND p.userID='.(int)$userID;
    
    $sql .= 'ORDER BY pos DESC';
    
    $result = $this->_db->select($sql);
    
    $list = array();
    
    if ($result)
    {
      $refs = array();
      
      while ($row = $this->_db->fetchAssoc($result))
      { 
        $thisref = &$refs[$row['postID']];
        
        $thisref['parentID'] = $row['parentID'];
        $thisref['postID'] = $row['postID'];
        $thisref['userID'] = $row['userID'];
        $thisref['categoryID'] = $row['categoryID'];
        $thisref['title'] = $row['title'];
        $thisref['description'] = $row['description'];
        $thisref['content'] = $row['content'];
        $thisref['image'] = $row['image'];
        $thisref['comments'] = $row['comments'];
        $thisref['url_tag'] = $row['url_tag'];
        $thisref['url'] = $row['url'];
        $thisref['pos'] = $row['pos'];
        $thisref['category'] = $row['category'];
        $thisref['status'] = $row['status'];
        $thisref['hidden'] = $row['hidden'];
        $thisref['userName'] = $row['userName'];
        $thisref['userUsername'] = $row['userUsername'];
          
        if (!$row['parentID'])
        {
          $list[$row['postID']] = &$thisref;
        }
        else
        {
          $refs[$row['parentID']]['children'][$row['postID']] = &$thisref;
        }
      }
    }
    return $list;
  }

  public function getPagesUl($before = null, $after = null, $replace = 'postID', $active = null, $pages = null)
  {
    static $init = false;
    if (!$init)
    {
      $pages = $this->getPages();
      $init = true;
    }
    
    $anchor = '';
    
    foreach ($pages as $postID => $page)
    {
      // Is page hidden from navigation?
      if ($page['hidden'] == 1) continue;
        
      $has_children = isset($page['children']);
      
      if ($active == $page['postID']/* || ($has_children && array_key_exists($active, $page['children']))*/)
        $class = ' class="ebl-current"';
      else
        $class = '';
      
      if ($page['url'])
        $anchor = '<a href="'.$page['url'].'">';
      else
        $anchor = '<a href="'.href('posts/view/'.$page['url_tag']).'"'.$class.'>';
        
      echo '<li',$class,'>',str_replace('%s', $page[$replace], $before),$anchor,$page['title'],'</a>',
        str_replace('%s', $page[$replace], $after);
      
      if ($has_children)
      {
        echo '<ul>';
        $this->getPagesUl($before, $after, $replace, $active, $page['children']);
        echo '</ul>';
      }
      
      echo '</li>';
    }
  }
  
  public function getPagesSelect($excludeID = 0, $indent = '', $active = null, $pages = null)
  {
    static $init = false;
    if (!$init)
    {
      $pages = $this->getPages();
      $init = true;
    }
      
    foreach ($pages as $postID => $page)
    {
      if ($postID == $excludeID)
        continue;
      
      $selected = '';
      if ($active == $page['postID'])
        $selected = 'selected="selected"';
      
      echo '<option value="',$postID,'"',$selected,'>',$indent,' ',$page['title'],'</option>';
      
      if (isset($page['children']))
      {
        $this->getPagesSelect($excludeID, $indent.'&mdash;', $active, $page['children']);
      }
    }
    $indent = '';
  }
  
  public function getLinks()
  {
    $sql = 'SELECT linkID, title, url, pos FROM '.DB_PREFIX.'menu_links ORDER BY pos ASC';
    $result = $this->_db->select($sql);
    
    $data = array();
    if ($result)
      while ($row = $this->_db->fetchAssoc($result))
        $data[$row['linkID']] = $row;
    return $data;
  }

  public function getLink($linkID)
  {
    $linkID = (int)$linkID;
    $sql = 'SELECT linkID, title, url, pos FROM '.DB_PREFIX.'menu_links
      WHERE linkID='.$linkID.' LIMIT 1';
    $result = $this->_db->select($sql);
    
    if ($result)
      return $this->_db->fetchAssoc($result);
    return null;
  }
  
  public function showLinks()
  {
    $links = $this->getLinks();
      
    foreach ($links as $linkID => $data)
    {
      echo '<li><a href="',$data['url'],'">',$data['title'],'</a></li>';
    }
  }
  
  public function showCategories()
  {
    if ($this->_categories === null)
      $this->_categories = $this->getCategories();
      
    foreach ($this->_categories as $categoryID => $data)
    {
      echo '<li><a href="',href('index/category/'.$data['url_tag'].'/'.$categoryID),'">',$data['title'],'</a></li>';
    }
  }
  
  public function showPosts($type = 'recent', $qty = 10)
  {
    $posts = $this->getPosts($type, $qty);
    foreach ($posts as $postID => $data)
      echo '<li><a href="',href('posts/view/'.$data['url_tag'].'/'.$data['postID']),'">',$data['title'],'</a></li>';
  }
  
  public function showArchives()
  {
    $sql = 'SELECT MONTHNAME(date_posted) AS
      `month`, YEAR(date_posted) AS `year`, count(postID) AS postsNum
      FROM '.DB_PREFIX.'posts
      WHERE status="approved" AND type="post"
      GROUP BY month ORDER BY date_posted DESC';
    $result = $this->_db->select($sql);
    if ($result)
    {
      while ($row = $this->_db->fetchAssoc($result))
        echo '<li><a href="',href('index/index/year/'.$row['year'].'/month/'.$row['month']),'">',__($row['month']),' ',$row['year'],'</a></li>';
    }
  }
  
  public function getComments($postID = 0, $status = 'approved', $format = 'ArrayAll', $page=0, $perpage=10, $order='asc')
  {
    $page = (int)$page;
    $postID = (int)$postID;
    if (!in_array($status, array('approved','pending','spam','deleted')))
      $status = null;
    
    $sql = 'SELECT c1.commentID, c1.parentID, c1.postID, c1.categoryID, IFNULL(u.name, c1.name) as name,
      IFNULL(u.email, c1.email) as email, c1.email_hide, c1.content, UNIX_TIMESTAMP(c1.date_posted) as date_posted, c1.userID,
      c1.ip, IFNULL(u.url, c1.website) as website, c1.status
      FROM '.DB_PREFIX.'comments c1
      LEFT JOIN '.DB_PREFIX.'users u ON u.userID = c1.userID
      WHERE 1';
      
    // Filter by postID
    $postID = (int)$postID;
    if ($postID)
      $sql .= ' AND c1.postID='.$postID;
      
    // Filter by status
    if ($status)
      $sql .= ' AND c1.status="'.$status.'"';
    
    $orderStr = null;
    if ($order == 'asc')
      $orderStr = ' ORDER BY c1.commentID ASC';
    elseif ($order == 'desc')
      $orderStr = ' ORDER BY c1.ommentID DESC';

    if ($format == 'ArrayAll')
    {
      $sql .= $orderStr;
      
      $result = $this->_db->select($sql);
      
      $refs = array();
      $list = array();
      
      if ($result)
      {
        while ($row = $this->_db->fetchAssoc($result))
        {
          $thisref = &$refs[$row['commentID']];
          
          $thisref['commentID'] = $row['commentID'];
          $thisref['parentID'] = $row['parentID'];
          $thisref['postID'] = $row['postID'];
          $thisref['categoryID'] = $row['categoryID'];
          $thisref['name'] = $row['name'];
          $thisref['email'] = $row['email'];
          $thisref['email_hide'] = $row['email_hide'];
          $thisref['content'] = $row['content'];
          $thisref['date_posted'] = $row['date_posted'];
          $thisref['userID'] = $row['userID'];
          $thisref['ip'] = $row['ip'];
          $thisref['website'] = $row['website'];
          $thisref['status'] = $row['status'];
            
          if (!$row['parentID'])
            $list[$row['commentID']] = &$thisref;
          else
            $refs[$row['parentID']]['children'][$row['commentID']] = &$thisref;
        }
      }
      return $list;
    }
    else if ($format == 'ResultByPage')
    {
      require_once APPLICATION.'core/pager.php';
      $pager = new Pager($page);
      $result = $db->query($sql);
      if ($result)
        $pager->total_items = $this->_db->numRows($result);
      $pager->total_pages = ceil($pager->total_items / $perpage);
      $offset = ($page - 1) * $perpage;
      $pager->items = $this->_db->select($sql.$orderStr." LIMIT $offset, $perpage");
      return $pager;
    }
    else
    {
      return $this->_db->select($sql);
    }
  }

  public function getComment($commentID)
  {
    $commentID = (int)$commentID;
    if (!$commentID)
      return null;
    
    $sql = 'SELECT c1.commentID, c1.parentID, c1.postID, c1.categoryID, c1.name,
      c1.email, c1.email_hide, c1.content, UNIX_TIMESTAMP(c1.date_posted) as date_posted, c1.userID,
      c1.ip, c1.website, c1.status
      FROM '.DB_PREFIX.'comments c1
      WHERE commentID='.$commentID.' LIMIT 1';
      
    $result = $this->_db->select($sql);
    if ($result)
      return $this->_db->fetchAssoc($result);
    return null;
  }

  public function getCommentsStats($userID = null)
  {
    $sql = 'SELECT
      SUM(IF(status<>"deleted",1,0)) as total,
      SUM(IF(status="pending",1,0)) as pending,
      SUM(IF(status="approved",1,0)) as approved,
      SUM(IF(status="spam",1,0)) as spam,
      SUM(IF(status="deleted",1,0)) as deleted
      FROM '.DB_PREFIX.'comments
      WHERE 1';
    if (is_numeric($userID))
      $sql .= ' AND userID='.$userID;
    $result = $this->_db->select($sql);
    if ($result)
      return $this->_db->fetchAssoc($result);
    return null;
  }
  
  public function getPostsStats($type = 'post', $userID = null)
  {
    if (!in_array($type, array('post','page')))
      $type = 'post';
      
    $sql = 'SELECT
      SUM(IF(status<>"deleted",1,0)) as total,
      SUM(IF(status="pending",1,0)) as pending,
      SUM(IF(status="approved",1,0)) as approved,
      SUM(IF(status="draft",1,0)) as draft,
      SUM(IF(status="deleted",1,0)) as deleted
      FROM '.DB_PREFIX.'posts WHERE type="'.$type.'"';
    if (is_numeric($userID))
      $sql .= ' AND userID='.$userID;
    $result = $this->_db->select($sql);
    if ($result)
      return $this->_db->fetchAssoc($result);
    return null;
  }
}