<?php

$usersM = Base::getModel('users');

switch ($request['action'])
{
  // SHOW USERS
	case 'index':
		usersAuth($usersM);
    usersCheckAccess('users_view');
		
		$filter = array();
		if (isset($_GET['groupID']) && is_numeric($_GET['groupID']))
			$filter['groupID'] = $_GET['groupID'];
    
    $page = isset($_GET['p']) ? $_GET['p'] : 1;
    $users = $usersM->getAll($page, 20, $filter);
    
    $groups = $usersM->getGroups();
    
    $title = $active = __('View users');
    break;
	
  // EDIT USER
  case 'edit':
		usersAuth($usersM);
    $hasAccess = $usersM->perm('users_edit');

		// Check permission
		if (!$hasAccess)
		{
		  // User can edit his/her own profile only
			$userID = $_SESSION['eblUserID'];
			$_POST['groupID'] = $_SESSION['eblGroupID'];
		}
    else if ($hasAccess)
    {
      if (isset($_GET['userID']))
        $userID = $_GET['userID'];
      else if (isset($_POST['userID']))
        $userID = $_POST['userID'];
      else
        $userID = 0;
    }
    else
    {
      set_flash_message(__('Access denied'), 'error');
      redirect(href('users/index'));
    }
		
		// Get user's info
		$userInfo = $usersM->get($userID);
		
		// Get groups
		$groups = $usersM->getGroups();
    
    // Get url to return to
    $from = getFromUrl();
    if (!$from)
      $from = href('users/index');
    
		$errors = '';
		$success = false;
		
		if (isset($_POST['userEdit']))
		{
			// VALIDATE
			if ($_POST['token'] != get_token())
				$errors .= '<li>'.__('Error occured. You may try again.').'</li>';
			
			if ($userInfo == false && !preg_match('/^.{8,20}$/', $_POST['password']))
				$errors .= '<li>'.sprintf(__('Password must be at least %d and maximum %d characters long'), 8, 20).'</li>';
				
			if ($_POST['password'] != $_POST['password2'])
				$errors .= '<li>'.__('Passwords do not match').'</li>';
	
			if (!preg_match('|^[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\.[A-Za-z]{2,4}$|u', $_POST['email']))
				$errors .= '<li>'.__('Enter correct email').'</li>';
			
      // Verify unique email
      $account_exists = $db->fetchResult('SELECT userID FROM '.DB_PREFIX.'users WHERE email="'.
        $db->escape($_POST['email']).'" LIMIT 1');
      if ($account_exists && $account_exists != $userID)
        $errors .= '<li>'.sprintf(__('Email %s is already in use'), filterXss($_POST['email'])).'</li>';
      
			if (!preg_match('|^[a-z0-9\-\_]{2,30}$|u', $_POST['username']))
				$errors .= '<li>'.__('Please enter username. Allowed characters: a-z 0-9 - _').'</li>';
				
			if (!preg_match('|^.{2,30}$|u', $_POST['name']))
				$errors .= '<li>'.__('Please enter name').'</li>';
			
			// Check if the chosen group exists
			if (array_key_exists($_POST['groupID'], $groups) === false)
				$errors .= '<li>'.__('Please select a group').'</li>';
				
			// FILTER
			$_POST['email'] = filterXss($_POST['email']);
			$_POST['username'] = filterXss($_POST['username']);
			$_POST['name'] = filterXss($_POST['name']);
			
			if (isset($_POST['active']) && $_POST['active'] == 'y')
				$_POST['active'] = 'y';
			else
				$_POST['active'] = 'n';
				
			if ($errors == '')
			{
				if ($userInfo == false)
				{
					// Prepare password
					$_POST['password'] = sha1($_POST['password'].Base::get('salt'));
					$success = $usersM->save($_POST);
				}
				else
				{
					// Prepare password
					if ($_POST['password'] != '')
						$_POST['password'] = sha1($_POST['password'].Base::get('salt'));
					else
						$_POST['password'] = $userInfo['password'];
					
					$success = $usersM->save($_POST);
				}
        
				if (!$success)
        {
					$errors .= '<li>Server side error occured, please try again</li>';
        }
				else
				{
				  // Create upload directory for the user
				  require_once APPLICATION.'core/files.php';
				  $path = Base::get('uploads_dir').'authors/'.filter_filename($success).'/';
				  if (!file_exists($path))
          {
            mkdir($path, 0755);
          }
				  
					set_flash_message(__('Saved'), 'success');
					redirect(getFromUrl());
				}
			}
		}
		else if ($userInfo)
		{
			$_POST = $userInfo;
		}
		else
		{
			$_POST = array('active' => 'y');
		}
    
    $title = $active = __('Create user');
	  break;
	
  // DELETE USER
	case 'delete':
    usersAuth($usersM);
    usersCheckAccess('users_delete');
    
    $result = null;
    
    if (Base::getRequest('token') == get_token())
    {
      $userID = (int)Base::getRequest('userID');
      $sql = 'DELETE FROM '.DB_PREFIX.'users WHERE userID='.$userID.' LIMIT 1';
      $result = $db->query($sql);
    }
    
    if (!$result)
      set_flash_message(__('Error occured'), 'error');
    else
      set_flash_message(__('Complete'), 'success');
      
    redirect(href('users/index'));
    break;
	
  // AUTHENTICATE USER
  case 'login':
		$errors = '';
		
		if (!$usersM->loggedIn())
		{
			if(isset($_POST['email']) && isset($_POST['password']))
			{
				if (!$usersM->login())
					$errors .= '<li class="warning">'.__('Invalid login details').'</li>';
				else
					redirect(href('index/index'));
			}
		}
		else
			redirect(href('index'));
      
    $title = $active = __('Log in');
    break;
	
  // LOG THE USER OUT
  case 'logout':
		$usersM->logout();
		redirect(href('users/login'));
    break;
}