<?php

class Users
{
	private $db;
	
	public function __construct($db)
	{
		$this->db = $db;
		$this->salt = Base::get('salt');
	}
	
	/*
	 * Authenticate the user
	 * @return boolean
	 */
	function login()
	{
		$sql = 'SELECT u.userID, u.groupID, u.username, u.name, u.email, ug.tag as groupTag
			FROM '.DB_PREFIX.'users u
			LEFT JOIN '.DB_PREFIX.'users_groups ug ON ug.groupID = u.groupID
			WHERE u.username = "'.$this->db->escape($_POST['email']).'"
			 AND u.password = "'.sha1($_POST['password'].$this->salt).'"
			 AND u.active = "y"
			LIMIT 1';
    
		$result = $this->db->select($sql);
		if (!$result)
			return false;
		
		$user = $this->db->fetchAssoc($result);
		
		// Save user's info to SESSION
		session_regenerate_id();
		$_SESSION['eblUserID'] = $user['userID'];
		$_SESSION['eblGroupID'] = $user['groupID'];
    $_SESSION['eblGroup'] = $user['groupTag'];
		$_SESSION['eblUsername'] = $user['username'];
		$_SESSION['eblName'] = $user['name'];
		$_SESSION['eblEmail'] = $user['email'];
		$_SESSION['eblHash'] = sha1($_SERVER['HTTP_USER_AGENT'].$user['email'].$this->salt);
		
		return true;
	}
	
	/*
	 * Log the user out
	 * @return void
	 */
	function logout()
	{
		unset($_SESSION['eblUserID']);
		unset($_SESSION['eblGroupID']);
		unset($_SESSION['eblUsername']);
		unset($_SESSION['eblName']);
		unset($_SESSION['eblEmail']);
		unset($_SESSION['eblHash']);
		
		// Delete session cookie
		if (ini_get("session.use_cookies"))
		{
			$params = session_get_cookie_params();
			setcookie(session_name(), '', time() - 42000, $params["path"], $params["domain"], $params["secure"], $params["httponly"]);
		}
	}
	
	/*
	 * Check if user is logged in
	 * @return boolean
	 */
	function loggedIn()
	{
		if (isset($_SESSION['eblHash']))
		{
			if (sha1($_SERVER['HTTP_USER_AGENT'].$_SESSION['eblEmail'].$this->salt) == $_SESSION['eblHash'])
				return true;
		}
		$this->logout();
		return false;
	}
	
	/*
	 * Check if the user has a permission
	 * @param string $permission
	 * @return bool
	 */
	function perm($permission, $compare = 'and')
	{
		static $perm = array();
		static $done = false;
		
		if ($done == false && $this->loggedIn())
    {
      // Get group's permissions
	    $sql = 'SELECT p.permissionID, p.name
			  FROM '.DB_PREFIX.'users_permissions p
        INNER JOIN '.DB_PREFIX.'users_groups2permissions g2p
          ON g2p.groupID='.intval($_SESSION['eblGroupID']).'
            AND g2p.permissionID=p.permissionID';
            
			$result = $this->db->query($sql);
			if ($result)
				while ($row = $this->db->fetchAssoc($result))
					$perm[$row['permissionID']] = $row['name'];		
			$done = true;
		}
		
		if (is_array($permission))
		{
			$matches = 0;
			$perms = count($permission);
			foreach ($permission as $val)
			{
				if (in_array($val, $perm))
				{
					if ($compare == 'or')
					  return true;
					++$matches; 
				}
			}
			if ($matches != 0 && $matches == $perms)
				return true;
		}
		else
    {
			return in_array($permission, $perm);
    }
		
		return false;
	}
	
	/*
	 * Get one user
	 * @param int $userID
	 * @return array on success | false on failure
	 */
	function get($userID)
	{
		$result = $this->db->query('SELECT * FROM '.DB_PREFIX.'users WHERE userID = '.intval($userID).' LIMIT 1');
		if ($result && $this->db->numRows($result) > 0)
			return $this->db->fetchAssoc($result);
			
		return null;
	}
	
	/*
	 * Get users by group
	 * @param mixed $groupName String or Array of strings
	 * @return array on success | false on failure
	 */
	function getByGroup($groupName, $format = 'assoc')
	{
		$groups = '';
		
		if (is_array($groupName))
		{
			foreach ($groupName as $group)
				$groups .= '"'.$this->db->escape($group).'",';
			$groups = substr($groups, 0, -1);
		}
		else
			$groups = '"'.$this->db->escape($groupName).'"';
		
		$sql = 'SELECT u.userID, u.username, u.name, u.email, u.url, g.name
		  FROM '.DB_PREFIX.'users u
			INNER JOIN '.DB_PREFIX.'users_groups g ON g.groupID = u.groupID AND g.tag IN ('.$groups.')';
		$result = $this->db->select($sql);
		
		$data = array();
		if ($result)
		{
			if ($format == 'assoc')
			{
				while ($row = $this->db->fetchAssoc($result))
					$data[$row['userID']] = $row;
			}
      else if ($format == 'selectBox')
      {
        while ($row = $this->db->fetchAssoc($result))
          $data[$row['userID']] = $row['username'];
      }
			else
			{
				return $result;
			}
		}
		return $data;
	}
	
	/*
	 * Get all users by page
	 * @param int $page
	 * @param int $perpage Number of users per page
	 * @param array $filter
	 */
	function getAll($page, $perpage = 20, $filter = null)
	{	
		if (!is_numeric($page)) $page = 1;
		
		Base::loadLib('pager');
		$pager = new Pager($page);
			
		$pager->total_items = $this->db->fetchResult('SELECT COUNT(1) FROM '.DB_PREFIX.'users');
		$pager->total_pages = ceil($pager->total_items / $perpage);
		$offset = ($page - 1) * $perpage;
		
		$sql = 'SELECT u.userID, u.groupID, u.username, u.name, u.email, u.url,
			u.password, u.active, g.name as groupName FROM '.DB_PREFIX.'users u
			INNER JOIN '.DB_PREFIX.'users_groups g ON g.groupID = u.groupID';
		
		if (is_array($filter))
		{
			if (isset($filter['groupID']) && is_numeric($filter['groupID']))
				$sql .= ' AND u.groupID='.$filter['groupID'];
		}
		
		$sql .= ' ORDER BY u.name LIMIT '.$offset.','.$perpage;
		
		$pager->items = $this->db->query($sql);
		
		return $pager;
	}
	
	/*
	 * Get all users groups
	 * @return array on success | false in failure
	 */
	function getGroups()
	{
		$sql = 'SELECT * FROM '.DB_PREFIX.'users_groups';
    $result = $this->db->select($sql);
    $groups = array();
    if ($result)
      while ($row = $this->db->fetchAssoc($result))
        $groups[$row['groupID']] = $row['name'];
    return $groups;
	}
	
	/*
	 * Get single group
	 * @param int $groupID
	 * @return array on success | false on failure
	 */
	function getGroup($groupID)
	{
		if (!is_numeric($groupID)) return null;
		$result = $this->db->select('SELECT * FROM '.DB_PREFIX.'users_groups WHERE groupID='.$groupID.' LIMIT 1');
		if ($result)
		{
			return $this->db->fetchAssoc($result);
		}
		return null;
	}
	
	/*
	 * Get all permissions
	 * @return array
	 */
	function getPermissions()
	{
		$sql = 'SELECT permissionID, name, description, type FROM '.DB_PREFIX.'users_permissions';
		
		$result = $this->db->select($sql);
		$perm = array();
		if ($result)
		{
			while ($row = $this->db->fetchAssoc($result))
				$perm[$row['type']][$row['permissionID']] = $row;
		}
		return $perm;
	}
	
  function getAttr($key, $userID)
  {
    $userID = (int)$userID;
    return $this->db->fetchResult('select '.$key.' from '.DB_PREFIX.' WHERE userID='.$userID.' LIMIT 1');
  }
  
	/*
	 * INSERT or UPDATE user data in the database
	 * @param array $data User's info
	 * @return bool
	 */
	function save($data)
	{
		$userExists = is_numeric($data['userID']) && $data['userID'] > 0;
		if ($userExists)
			$sql = 'UPDATE '.DB_PREFIX.'users SET groupID = %d, email = "%s", password = "%s", username = "%s",
				name = "%s", url="%s", active = "%s" WHERE userID = '.$data['userID'];
		else
			$sql = 'INSERT INTO '.DB_PREFIX.'users (groupID, email, password, username, name, url, active)
				VALUES (%d, "%s", "%s", "%s", "%s", "%s", "%s")';
		
		$result = $this->db->query(sprintf($sql, intval($data['groupID']), $this->db->escape($data['email']),
			$data['password'], $this->db->escape($data['username']), $this->db->escape($data['name']),
			$this->db->escape($data['url']), $this->db->escape($data['active'])));
    
		if ($result)
		{
		  if (!$userExists) $userID = $this->db->getInsertId();
      else $userID = $data['userID'];
      
			if ($userExists)
				do_action('users_save_after', $userID);
			else
				do_action('users_save_after', $this->db->getInsertId());
        
			return $userID;
		}
    return false;
	}
}