<?php
require_once '../../config.php';
Base::set('db', new Db(Base::get('db_default')));

$usersM = Base::getModel('users');
if (!$usersM->loggedIn())
  exit(); // verify access
  
?>
<!DOCTYPE html>
<html>
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
	<title>File manager</title>
	
	<style type="text/css">
	/* v1.0 | 20080212 */
	html, body, div, span, applet, object, iframe,
	h1, h2, h3, h4, h5, h6, p, blockquote, pre,
	a, abbr, acronym, address, big, cite, code,
	del, dfn, em, font, img, ins, kbd, q, s, samp,
	small, strike, strong, sub, sup, tt, var,
	b, u, i, center,
	dl, dt, dd, ol, ul, li,
	fieldset, form, label, legend,
	table, caption, tbody, tfoot, thead, tr, th, td {
		margin: 0;
		padding: 0;
		border: 0;
		outline: 0;
		font-size: 100%;
		vertical-align: baseline;
		background: transparent;
	}
	body {
		line-height: 1;
	}
	ol, ul {
		list-style: none;
	}
	blockquote, q {
		quotes: none;
	}
	blockquote:before, blockquote:after,
	q:before, q:after {
		content: '';
		content: none;
	}
	
	/* remember to define focus styles! */
	:focus {
		outline: 0;
	}
	
	/* remember to highlight inserts somehow! */
	ins {
		text-decoration: none;
	}
	del {
		text-decoration: line-through;
	}
	
	/* tables still need 'cellspacing="0"' in the markup */
	table {
		border-collapse: collapse;
		border-spacing: 0;
	}
	
	/* Various elements */
	body { color: #777; font-size: 12px; font-family: Arial, Helvetica, sans-serif; padding: 10px; }
	
	a { color: #679BCF; text-decoration: none; }
	a:hover { color: #333; }
	.wrap { height: 300px; margin-bottom: 20px; overflow-y: scroll; width: 100%; }
	
	h4 { margin-bottom: 15px; }
	p { margin-bottom: 15px; }
	
	.button { background-color: #eee; border: 1px solid #dfdfdf; color: #777; padding: 2px 5px; }
	.button:hover { border: 1px solid #b0b0b0; color: #444; cursor: pointer; }
	
	input, select, textarea { border: 1px solid #bfbfbf; padding: 2px 5px; }
	
	/* Message box */
	#messageBox { background-color: #f5f5f5; border: 1px dotted #eee; margin-bottom: 10px; padding: 5px; }
	
	/* Ajax loader */
	.ajax_loader { background: url('images/ajax-loader.gif') no-repeat center center; }
	
	/* Browser */
	#browser { background: url('images/table-header.png') repeat-x top; border-collapse: collapse; width: 100%; }
	#browser tr:hover td { background-color: #f0f0f0; }
	#browser td { background: #f5f5f5 url('images/line-h.gif') repeat-x bottom; padding: 5px 10px 10px 10px; }
	#browser td.icon img { display: inline; vertical-align: top; }
	#browser th { border: 0; color: #444; font-weight: normal; padding: 7px 10px 7px 10px; height: 15px; text-align: left; }
	
	/* Forms */
	#forms { height: 250px; overflow: hidden; }
	#forms .tab { height: 250px; }
	#forms label { display: inline-block; width: 100px; }
	
	/* Options plugin */
	.options { position: relative; }
	.options ul { background-color: #eee; position: absolute; top: 0; left: 0; z-index: 20; }
	.options ul li { list-style: none; }
	.options ul li a { color: #777; display: block; padding: 4px 8px; text-decoration: none; width: 100px; }
	.options ul li a:hover { background-color: #e1e1e1; }
	
	</style>
	
	<script type="text/javascript" src="js/jquery.min.js"></script>
	<script type="text/javascript">
	/*
	 * Message box
	 */
	var DmMessageBox =
	{
		messageBox: null,
		timeout: null,

		init: function(boxID)
		{
			this.messageBox = $(boxID);
			this.messageBox.fadeOut('fast');
		},
		
		set: function(msg)
		{
			this.messageBox.html(msg);
			this.messageBox.fadeIn('slow');
			this.hide();
		},

		hide: function()
		{
			var self = this;
			clearTimeout(this.timeout);
			this.timeout = setTimeout(function() { self.messageBox.fadeOut('slow'); }, 10000);
		}
	}
	
	/* 
	 * File Manager
	 */
	var DmFileManager =
	{
		init: function(browserID, formFileID, messageBoxID)
		{
			this.browser = $(browserID);
			this.formFile = $(formFileID);
			this.currDir = this.formFile.children('input:first');
			this.newDirName = document.getElementById('newDirName');
			this.selected = '';

			// Insert form
			this.fileInsertButton = $('#fileInsertButton');
			this.fileInsertFileName = $('#fileInsertFileName');
			this.fileInsertFileUrl = $('#fileInsertFileUrl');
			this.fileInsertContent = $('#fileInsertContent');

			var that = this;
			
			this.fileInsertButton.bind('click', function()
			{
				window.opener.DmFileManagerCallback({
					name: that.fileInsertFileName.text(),
					url: that.fileInsertFileUrl.text(),
					content: that.fileInsertContent.val()
				});
			});
	
			this.changeDir('');
		},

		showLoader: function()
		{
			// Hide browser and show loader
			this.browser.css('visibility', 'hidden');
			this.browser.parent().addClass('ajax_loader');
		},

		hideLoader: function()
		{
			// Hide loader and show browser
			this.browser.css('visibility', 'visible');
			this.browser.parent().removeClass('ajax_loader');
		},
		
		changeDir: function(dir)
		{
			this.showLoader();
			
			var self = this;
			$.ajax({
				cache: false,
				type: 'GET',
				url: 'manager.php',
				data: 'action=dirView&dir='+dir,
				success: function(response)
				{
					self.browser.html(response);
					self.currDir.val(dir);
					self.hideLoader();
				}
			});
		},

		uploadStart: function(el)
		{
			this.formFile.submit();
			el.disabled = true;
		},
		
		formFileCallback: function(response)
		{
			if (response)
			{
				this.changeDir(this.currDir.val());
				DmMessageBox.set(response);
			}
			else
				DmMessageBox.set(response);

			this.formFile.find("button:first").removeAttr("disabled");
		},

		fileDelete: function(filename)
		{
			if (!confirm('You are about to delete a file'))
				return false;
			this.showLoader();
			var self = this;
			$.ajax({
				cache: false,
				type: 'GET',
				url: 'manager.php',
				data: 'action=fileDelete&dir='+self.currDir.val()+'&filename='+filename,
				success: function(response)
				{
					if (response == 1)
					{
						DmMessageBox.set('File has been deleted');
						self.changeDir(self.currDir.val()); // refresh current directory
					}
					else
						DmMessageBox.set('Error occured. File has not been deleted');					
					self.hideLoader();
				}
			});
			return false;
		},

		dirDelete: function(dirname)
		{
			if (!confirm('You are about to delete a directory'))
				return false;
			this.showLoader();
			var self = this;
			$.ajax({
				cache: false,
				type: 'GET',
				url: 'manager.php',
				data: 'action=dirDelete&dir='+self.currDir.val()+'&dirname='+dirname,
				success: function(response)
				{
					if (response == 1)
					{
						DmMessageBox.set('Directory has been deleted');
						self.changeDir(self.currDir.val()); // refresh current directory
					}
					else
						DmMessageBox.set('Error occured. Please make sure that directory is empty');
					self.hideLoader();
				}
			});
			return false;
		},

		dirCreate: function()
		{
			this.showLoader();
			var self = this;
			$.ajax({
				cache: false,
				type: 'POST',
				url: 'manager.php?action=dirCreate&dir='+self.currDir.val(),
				data: 'dirname='+self.newDirName.value,
				success: function(response)
				{
					if (response == '1')
					{
						DmMessageBox.set('New directory has been created');
						self.changeDir(self.currDir.val()); // refresh current directory
						self.newDirName.value = '';
					}
					else
					{
						DmMessageBox.set('Error occured. Check that directory does not exist already');
					}
					
					self.hideLoader();
				}
			});
			return false;
		},

		windowCallback: function(type, data)
		{
			var self = this;
			
			switch (type)
			{
				case 'rename':
					$.ajax({
						type: 'POST',
						url: 'manager.php?action=rename&type='+data.type+'&dir='+self.currDir.val(),
						data: { oldname: data.oldName, newname: data.newName },
						success: function(response)
						{
							if (response == '1')
							{
								self.changeDir(self.currDir.val()); // refresh current directory
								self.selected = null;
							}
						}
					});
					break;
			}
		},

		rename: function(type, name)
		{
			var currDir = this.currDir.val();
			if (currDir.substring(0,-1) != "/") currDir += "/";
			this.selected = currDir;
			var newWin = window.open('rename.html', 'Rename', 'height=60,width=400,toolbar=no,directories=no,status=no,menubar=no,scrollbars=no,resizable=no');

			// Pass data to the child window
			function waitForWin()
			{
				if (newWin.data)
				{
					newWin.data.oldName = name;
					newWin.data.type = type;
					newWin.setPrevName(type, name);
				}
				else
					setTimeout(waitForWin, 300);
			}
			waitForWin();
		},
		
		fileInsert: function(fileID)
		{
			var fileRow = $("#"+fileID).children("td");
			this.fileInsertFileName.text(fileRow.eq(1).text());
			this.fileInsertFileUrl.text(fileRow.eq(2).text());
			this.fileInsertContent.val("");
			window.location = "#fileInsert"; // Show form
		},

		fileInsertFormat: function(type)
		{
			var name = this.fileInsertFileName.text();
			var url = this.fileInsertFileUrl.text();
			
			var html;
			if (type == "link")
				html = "<a href=\""+url+"\">"+name+"</a>";
			else
				html = "<img src=\""+url+"\" alt=\""+name+"\" />";
							
			this.fileInsertContent.val(html);
		}
	};
	
	/*
	 * Options menu
	 */
	function dmOptionsShow(el)
	{
		var el = $(el);
		var options = el.data('options');
		if (!options)
		{
			options = el.children('ul:first');
			el.data('options', options);
		}
		options.css('display', 'block');
		return false;
	}
	
	function dmOptionsHide(el)
	{
		var el = $(el);
		var options = el.data('options');
		if (!options)
		{
			options = el.children('ul:first');
			el.data('options', options);
		}
		options.css('display', 'none');
		return false;
	}
	</script>
</head>
<body>

<div id="messageBox"></div>

<?php if ($usersM->perm('files_edit')): ?>
<p>
	<label>New directory</label>
	<input type="text" name="dirname" id="newDirName" value="" />
	<button type="submit" class="button" onclick="DmFileManager.dirCreate(); return false;">Create</button>
</p>
<?php endif; ?>

<div class="wrap">
	<table id="browser">
	</table>
</div>

<p>
	<a class="button" href="#formFile" title="Hi Im a tooltip!">Upload file</a> | <a class="button" href="#fileInsert">Insert file</a>
</p>

<div class="wrap" id="forms">
	<iframe name="formFileSubmit" style="width:0;height:0;visibility:hidden;"></iframe>
	<form class="tab" action="manager.php?action=upload" method="post" id="formFile"
		target="formFileSubmit" enctype="multipart/form-data">
		<h4>Upload files</h4>
		<input type="hidden" name="dir" value="" />
		<p>
			<label>File</label>
			<input type="file" name="file" />
			<button type="submit" class="button" onclick="DmFileManager.uploadStart(this);">Submit</button>
		</p>
	</form>
	
	<div id="fileInsert" class="tab">
		<h4>Prepare file</h4>
		<p>
			<label>Filename</label>
			<span id="fileInsertFileName"></span>
		</p>
		<p>
			<label>File url</label>
			<span id="fileInsertFileUrl"></span>
		</p>
		<p>
			<label>Select format</label>
			<label><input type="radio" name="fileInsertFormat" value="0" onclick="DmFileManager.fileInsertFormat('link');" /> Link</label>
			<label><input type="radio" name="fileInsertFormat" value="1" onclick="DmFileManager.fileInsertFormat('image');" /> Image</label>
		</p>
		<p>
			<label></label>
			<textarea id="fileInsertContent" rows="3" cols="80"></textarea>
		</p>
		<p>
			<button id="fileInsertButton" class="button">Insert</button>
		</p>
	</div>
</div>

<script type="text/javascript">
DmMessageBox.init('#messageBox');
DmFileManager.init('#browser', '#formFile');
</script>

</body>
</html>