<?php

// Include app files and check access
require_once '../../config.php';

// Connect to database
Base::set('db', new Db(Base::get('db_default')));

// Load theme plugin
$themeFunctions = THEMES.get_option('theme').'/functions.php';
if (file_exists($themeFunctions))
  require_once $themeFunctions;

// Locale
i18nInit($locale);

$users = Base::getModel('users');
if (!$users->loggedIn())
	exit(); // verify access

/*
 * FILE MANAGER 
 */

// Include functions
require_once 'files.php';
require_once 'lib.manager.php';

// Setup settings
$options = array();

// Get relative path to directory
if (isset($_GET['dir']))
	$options['requestDir'] = $_GET['dir'];
else if (isset($_POST['dir']))
	$options['requestDir'] = $_POST['dir'];
else
	$options['requestDir'] = '';

// Path and URL to the viewable directory
$options['filesDir'] = Base::get('uploads_dir');
$options['filesUrl'] = Base::get('uploads_url');

if ($_SESSION['eblGroup'] != 'admin')
{
  $authorPath = 'authors/'.$_SESSION['eblUserID'].'/';
  $options['filesDir'] .= $authorPath;
  $options['filesUrl'] .= $authorPath;
}

// Url to the root directory of the file manager
$options['managerUrl'] = 'http://'.$_SERVER['SERVER_NAME'].dirname($_SERVER['PHP_SELF']).'/';

// Prepare FileManager object
$fileManager = new FileManager($options);

// Get action
$action = isset($_GET['action']) ? $_GET['action'] : '';

switch($action)
{
	case 'dirView':
		// Read directory to html
		echo $fileManager->dirView();
		break;
		
	case 'upload':
		$msg = '';
		$result = $fileManager->fileUpload();		
		switch ($result)
		{
			case 'error_filetype':
				$msg = 'The type of your file is not supported by the system';
				break;
			case 'error_filename':
				$msg = 'The filename is invalid';
				break;
			case 'error_filesize':
				$msg = 'Your file is too big. Maximum file size is '.$fileManager->getMaxFileSize('kb').' kb';
				break;
			case 'error_fileempty':
				$msg = 'The file was empty or too large. Please refer to upload_max_filesize option in php.ini file';
				break;
			case 'success':
				$msg = 'The file has been uploaded';
				break;
			case 'failure':
				$msg = 'Internal error occured. Please try again';
				break;
			case 'error_fileexists':
				$msg = 'File with such name already exists';
				break;
		}
		echo '<script type="text/javascript">window.top.window.DmFileManager.formFileCallback(\''.$msg.'\');</script>';
		break;
	
	case 'fileDelete':
		$result = $fileManager->fileDelete();
		echo $result;
		break;
		
	case 'dirCreate':
		$result = $fileManager->dirCreate(0755);
		echo $result;
		break;
		
	case 'dirDelete':
		$result = $fileManager->dirDelete();
		echo $result;
		break;
		
	case 'rename':
		$result = $fileManager->rename();
		echo $result;
		break;
}

?>