/*
 * jQuery dmModal plugin
 * Author: Dmytro Danylov
 */
(function($)
{  
  function dmModal(el, options)
  {
    this.options = $.extend({
      buttons: null
    }, options);
    
    this.trigger = $(el);
    this.fade = $('#fade');
    this.modalID = this.trigger.attr('rel');
    this.modalBox = $('#' + this.modalID);
    this.args = new Array();
    this.hasButtons = false;
    this.buttons = null;
    
    // Pull Arguments from href
    var modalArgsStr = this.trigger.attr('href').substring(1);
    var argsTmp = modalArgsStr.split('&');
    for (var i = 0; i < argsTmp.length; ++i)
    {
      var arg = argsTmp[i].split('=');
      this.args[arg[0]] = arg[1];
    }
    
    this.modalWidth = Number(this.args['w']);
    
    var that = this;
    this.trigger.bind('click', function(e)
    {
      e.preventDefault();
      that.modalShow();
    });
    
    //Close Modal boxes
    $('.modal-close, #fade').bind('click', function(e)
    {
      e.preventDefault();
      that.modalClose();
    });
  };
  
  dmModal.prototype.modalShow = function()
  { 
    var that = this;
    
    // Append buttons
    if (this.options.buttons != null && this.hasButtons == false)
    {
      this.buttons = $('<div class="modal-buttons" />');
      this.modalBox.append(this.buttons);
      
      for (var i = 0; i < this.options.buttons.length; ++i)
      {
        button = this.options.buttons[i];
        var btn = $('<input type="submit" value="'+button.title+'" />');
        
        btn.data('idx', i);
        
        btn.bind('click', function(e)
        {
          e.preventDefault();
          that.modalClose();
          that.options.buttons[$(this).data('idx')].callback.call(that);
        });
        
        this.buttons.append(btn);
      }
      
      this.hasButtons = true;
    }
    else
      this.buttons.show();
    
    // Fade in the modal box and add close button
    this.modalBox.fadeIn().css({'width': this.modalWidth});

    var padTop = parseInt(this.modalBox.css('padding-top'));
    var padBottom = parseInt(this.modalBox.css('padding-bottom'));
    var modalMargTop = (this.modalBox.height() + padTop + padBottom) / 2;
    var modalMargLeft = (this.modalBox.width()) / 2;

    // Apply margin to modal box
    this.modalBox.css({
      'margin-top' : -modalMargTop,
      'margin-left' : -modalMargLeft
    });

    // fade in background
    this.fade.css({'filter': 'alpha(opacity=80)'}).fadeIn();
  };
  
  dmModal.prototype.modalClose = function()
  {
    var that = this;
    $('#fade, .modal-box').fadeOut(function() { if (that.buttons) that.buttons.hide(); });
  };
  
  $.fn.dmModal = function(options)
  {
    $('body').append('<div id="fade"></div>');
    
    return this.each(function()
    {
      var obj = new dmModal(this, options);
    });
  };
})(jQuery);